#!/usr/bin/python

import click
import unittest
import sys

from ConfigParser import ConfigParser

from unittest.runner import TextTestRunner

from chisubmit.tests import api
from chisubmit.tests import client
from chisubmit.tests import cli
from chisubmit.tests.integration.test_complete1 import CLICompleteWorkflowExtensionsPerTeam
from chisubmit.tests.integration.test_complete2 import CLICompleteWorkflowExtensionsPerStudent
from chisubmit.tests.integration.test_complete3 import CLICompleteWorkflowCancelSubmission

api_tests = unittest.TestLoader().loadTestsFromModule(api)
client_tests = unittest.TestLoader().loadTestsFromModule(client)
cli_tests = unittest.TestLoader().loadTestsFromModule(cli)

unit_tests = {"api": api_tests,
              "clientlibs": client_tests,
              "cli": cli_tests}

all_unittests = unittest.TestSuite(unit_tests.values())

integration_tests = {"complete1": CLICompleteWorkflowExtensionsPerTeam,
                     "complete2": CLICompleteWorkflowExtensionsPerStudent,
                     "complete3": CLICompleteWorkflowCancelSubmission}

@click.command()
@click.option("--failfast", "-f", is_flag=True)
@click.option("--quiet", "-q", is_flag=True)
@click.option("--verbose", "-v", is_flag=True)
@click.option("--buffer", "-b", is_flag=True)
@click.option("--integration-config", "-c", type=click.File())
@click.option("--integration-git-server", type=str)
@click.option("--integration-git-staging", type=str)
@click.argument('tests', type=str, default="all_unit")
def runtests(failfast, quiet, verbose, buffer, 
             integration_config, integration_git_server, integration_git_staging, 
             tests):
    verbosity = 1
    if quiet:
        verbosity = 0
    if verbose:
        verbosity = 2
        
    if integration_config is not None:
        config = ConfigParser()
        config.readfp(integration_config)
    else:
        config = None
        
    runner = TextTestRunner(verbosity=verbosity, failfast=failfast, buffer=buffer)
    
    ran = False
    
    if tests in ("all", "all_unit"):
        ran = True
        runner.run(all_unittests)
        
    if tests in unit_tests:
        ran = True
        runner.run(unit_tests[tests])
        
    if tests in integration_tests:
        ran = True
        test_class = integration_tests[tests]
        suite = unittest.TestSuite()
        for name in unittest.TestLoader().getTestCaseNames(test_class):
            test = test_class(name)
            configure_integration_test(test, config, integration_git_server, integration_git_staging)
            suite.addTest(test)
        
        runner.run(suite)
        
    if not ran:
        # TODO: Try to instantiate specific test
        try:
            suite = unittest.TestLoader().loadTestsFromName(tests)
            runner.run(suite)
        except AttributeError, ae:
            print "Unknown test: %s" % tests
    
    
def configure_integration_test(test, config, git_server, git_staging):
    if config is None and (git_server is not None or git_staging is not None):
        print "ERROR: You have specified a Git server or staging server, but have"
        print "       not provided a configuration file."
        sys.exit(1)
        
    if config is None:
        return
    
    if git_server is not None:
        if not config.has_section(git_server):
            print "ERROR: You specified a %s server, but the configuration file" % git_server
            print "       doesn't have a [%s] section" % git_server
        
        connstr = config.get(git_server, "server-connstr")
        apikey = config.get(git_server, "api-key")
        username = config.get(git_server, "username")
        
        test.set_git_server_connstr(connstr)
        test.add_api_key(git_server, apikey)
        test.set_git_server_user(username)
        
    if git_staging is not None:
        if not config.has_section(git_staging):
            print "ERROR: You specified a %s server, but the configuration file" % gitgit_staging_server
            print "       doesn't have a [%s] section" % git_staging
        
        connstr = config.get(git_staging, "staging-connstr")
        apikey = config.get(git_staging, "api-key")
        username = config.get(git_server, "username")
        
        test.set_git_staging_connstr(connstr, username)
        test.add_api_key(git_staging, apikey)
        test.set_git_staging_user(username)
            
if __name__ == "__main__":
    runtests()
    


from chisubmit.backend.webapp.api import db
from chisubmit.backend.webapp.api.models.mixins import ExposedModel
from chisubmit.backend.webapp.api.models.json import Serializable
from sqlalchemy.ext.associationproxy import association_proxy
from chisubmit.backend.webapp.api.types import JSONEncodedDict


class CoursesGraders(Serializable, db.Model):
    __tablename__ = 'courses_graders'
    repo_info = db.Column(JSONEncodedDict, default={})
    grader_id = db.Column('grader_id',
                          db.Unicode,
                          db.ForeignKey('users.id'), primary_key=True)
    course_id = db.Column('course_id',
                          db.Integer,
                          db.ForeignKey('courses.id'), primary_key=True)
    grader = db.relationship("User")
    
    # Dirty hack
    conflicts = db.Column(db.Unicode)
    
    default_fields = ['grader', 'repo_info', 'conflicts']
    readonly_fields = ['grader_id', 'course_id', 'repo_info']        
    course = db.relationship("Course",
                             backref=db.backref("courses_graders",
                                                cascade="all, delete-orphan"))


class CoursesStudents(Serializable, db.Model):
    __tablename__ = 'courses_students'
    repo_info = db.Column(JSONEncodedDict, default={})
    dropped = db.Column('dropped', db.Boolean, server_default='0',
                        nullable=False)
    extensions = db.Column(db.Integer, default=0)    

    student_id = db.Column('student_id',
                           db.Unicode,
                           db.ForeignKey('users.id'),
                           primary_key=True)
    course_id = db.Column('course_id',
                          db.Integer,
                          db.ForeignKey('courses.id'),
                          primary_key=True)
    student = db.relationship("User")
    default_fields = ['student', 'dropped', 'repo_info']
    readonly_fields = ['student_id', 'course_id', 'repo_info']    
    course = db.relationship("Course",
                             backref=db.backref("courses_students",
                                                cascade="all, delete-orphan"))

    @staticmethod
    def from_id(course_id, student_id):
        return CoursesStudents.query.filter_by(student_id=student_id, course_id=course_id).first()

    def get_extensions_available(self):
        from chisubmit.backend.webapp.api.teams.models import Team

        extensions_used = 0
        
        student_teams = Team.find_teams_with_students(self.course_id, [self.student])
        for student_team in student_teams:
            for assignment_team in student_team.assignments_teams:
                extensions_used += assignment_team.extensions_used

        return self.extensions - extensions_used
                

class CoursesInstructors(Serializable, db.Model):
    __tablename__ = 'courses_instructors'
    repo_info = db.Column(JSONEncodedDict, default={})
    instructor_id = db.Column('instructor_id',
                              db.Unicode,
                              db.ForeignKey('users.id'),
                              primary_key=True)
    course_id = db.Column('course_id',
                          db.Integer,
                          db.ForeignKey('courses.id'),
                          primary_key=True)
    instructor = db.relationship("User")
    default_fields = ['instructor', 'repo_info']
    readonly_fields = ['instructor_id', 'course_id', 'repo_info']    
    course = db.relationship("Course",
                             backref=db.backref("courses_instructors",
                                                cascade="all, delete-orphan"))


class Course(ExposedModel, Serializable):
    __tablename__ = 'courses'
    id = db.Column(db.Unicode, primary_key=True, unique=True)
    name = db.Column(db.Unicode)
    options = db.Column(JSONEncodedDict, default={})
    graders = association_proxy('courses_graders', 'grader')
    students = association_proxy('courses_students', 'student')
    instructors = association_proxy('courses_instructors', 'instructor')
    assignments = db.relationship("Assignment", backref="course")
    teams = db.relationship("Team", backref="course")
    default_fields = ['name', 'assignments', 'options']
    readonly_fields = ['id', 'options', 'graders', 'students', 'instructors',
                       'assignments', 'teams']
    
    @staticmethod
    def from_id(course_id):
        return Course.query.filter_by(id=course_id).first()
        



from chisubmit.tests.common import cli_test, ChisubmitIntegrationTestCase
from chisubmit.common.utils import get_datetime_now_utc, convert_datetime_to_local,\
    set_testing_now
from chisubmit.client.session import BadRequestError
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL

from datetime import timedelta
import os

class CLICompleteWorkflowExtensionsPerTeam(ChisubmitIntegrationTestCase):
        
    @cli_test
    def test_complete_with_extensions_per_team(self, runner):
        course_id = u"cmsc40200"
        course_name = u"Foobarmentals of Foobar"

        admin_id = u"admin"
        instructor_ids = [u"instructor"]
        grader_ids= [u"grader"]
        student_ids = [u"student1", u"student2", u"student3", u"student4"]
        
        all_users = instructor_ids + grader_ids + student_ids
        
        admin, instructors, graders, students = self.create_clients(runner, course_id, admin_id, instructor_ids, grader_ids, student_ids)
        self.create_users(admin, all_users)
        
        self.create_course(admin, course_id, course_name)

        result = admin.run("admin course set-option %s default-extensions 2" % (course_id))
        self.assertEquals(result.exit_code, 0)
        
        result = admin.run("admin course set-option %s extension-policy per_team" % (course_id))
        self.assertEquals(result.exit_code, 0)
        
        self.add_users_to_course(admin, course_id, instructors, graders, students)
        
        teams = ["the-flaming-foobars", "the-magnificent-mallocs"]        

        students_team = [students[0:2], students[2:4]]

        
        deadline = get_datetime_now_utc() - timedelta(hours=23)
        deadline = convert_datetime_to_local(deadline)
        deadline = deadline.replace(tzinfo=None).isoformat(sep=" ")        

        result = instructors[0].run("instructor assignment add", 
                                    ["pa1", "Programming Assignment 1", deadline])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor assignment add-grade-component", 
                                    ["pa1", "tests", "The PA1 Tests", "50"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor assignment add-grade-component", 
                                    ["pa1", "design", "The PA1 Design", "50"])
        self.assertEquals(result.exit_code, 0)

        deadline = get_datetime_now_utc() - timedelta(hours=49)
        deadline = deadline.isoformat(sep=" ")

        result = instructors[0].run("instructor assignment add", 
                                    ["pa2", "Programming Assignment 2", deadline])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor assignment add-grade-component", 
                                    ["pa2", "tests", "The PA2 Tests", "50"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor assignment add-grade-component", 
                                    ["pa2", "design", "The PA2 Design", "50"])
        self.assertEquals(result.exit_code, 0)
        
        
        result = admin.run("admin course show", ["--include-users", "--include-assignments", course_id])
        self.assertEquals(result.exit_code, 0)

        
        self.register_team(students_team[0], teams[0], "pa1", course_id)
        self.register_team(students_team[1], teams[1], "pa1", course_id)

        self.register_team(students_team[0], teams[0], "pa2", course_id)

        
        result = students_team[0][0].run("student team list")
        self.assertEquals(result.exit_code, 0)
        self.assertIn(teams[0], result.output)
        self.assertNotIn(teams[1], result.output)

        result = students_team[1][0].run("student team list")
        self.assertEquals(result.exit_code, 0)
        self.assertIn(teams[1], result.output)
        self.assertNotIn(teams[0], result.output)
        
        result = students_team[0][0].run("student team show", [teams[1]])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)        

        result = students_team[1][0].run("student team show", [teams[0]])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)        


        result = instructors[0].run("instructor team set-attribute", [teams[0], "alias", "foobar "+teams[0]])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor team set-attribute", [teams[1], "alias", "foobar "+teams[1]])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor team list")
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor team show", [teams[0]])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor team show", [teams[1]])
        self.assertEquals(result.exit_code, 0)

        
        team_git_paths, team_git_repos, team_commits = self.create_team_repos(admin, course_id, teams, students_team)
        
        # Try to submit without enough extensions
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][0].hexsha, 
                                          "--extensions", "0",
                                          "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)
        
        # Try to submit with too many extensions
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][0].hexsha, 
                                          "--extensions", "2",
                                          "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)

        # Submit with just the right number
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][0].hexsha, 
                                          "--extensions", "1",
                                          "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_SUCCESS)

        result = students_team[0][0].run("student team show", [teams[0]])
        self.assertEquals(result.exit_code, 0)

        # Try submitting an already-submitted assignment
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][1].hexsha, 
                                          "--extensions", "1",
                                          "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)

        # Try submitting an already-submitted assignment, with the same
        # commit as before 
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][0].hexsha, 
                                          "--extensions", "1",
                                          "--yes", "--force"])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)


        # Submit an already-submitted assignment
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa1", team_commits[0][1].hexsha, 
                                          "--extensions", "1",
                                          "--yes", "--force"])
        self.assertEquals(result.exit_code, CHISUBMIT_SUCCESS)        
        
        result = students_team[0][0].run("student team show", [teams[0]])
        self.assertEquals(result.exit_code, 0)
        
        # Try requesting more extensions than the team has
        result = students_team[0][0].run("student assignment submit", 
                                         [teams[0], "pa2", team_commits[0][1].hexsha, 
                                          "--extensions", "3",
                                          "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)
        
        # Try submitting for a project the team is not registered for
        result = students_team[1][0].run("student assignment submit", 
                                         [teams[1], "pa2", team_commits[1][1].hexsha, 
                                          "--extensions", "0",
                                          "--yes"])        
        self.assertEquals(result.exit_code, CHISUBMIT_FAIL)

        result = students_team[1][0].run("student assignment submit", 
                                         [teams[1], "pa1", team_commits[1][1].hexsha, 
                                         "--extensions", "1",
                                         "--yes"])
        self.assertEquals(result.exit_code, CHISUBMIT_SUCCESS)

        result = instructors[0].run("instructor grading list-submissions", ["pa1"])
        self.assertEquals(result.exit_code, 0)

        # Let the deadline "pass"
        new_now = get_datetime_now_utc() + timedelta(hours=2)
        set_testing_now(new_now)

        print
        print "~~~ Time has moved 'forward' by two hours ~~~"
        print

        result = instructors[0].run("instructor grading list-submissions", ["pa1"])
        self.assertEquals(result.exit_code, 0)
                
        result = instructors[0].run("instructor grading create-grading-repos", ["pa1"])
        self.assertEquals(result.exit_code, 0)        
        
        result = instructors[0].run("instructor grading create-grading-branches", ["pa1"])
        self.assertEquals(result.exit_code, 0)        
                
        result = instructors[0].run("instructor grading set-grade", 
                                [teams[0], "pa1", "tests", "100"])
        self.assertEquals(result.exit_code, 1)

        result = instructors[0].run("instructor grading set-grade", 
                                [teams[0], "pa1", "tests", "40"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading set-grade", 
                                [teams[1], "pa1",  "tests", "45"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading list-grades")
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading set-grade", 
                                [teams[0], "pa1", "tests", "50"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading list-grades")
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading add-rubrics", ["pa1", "--commit"])
        self.assertEquals(result.exit_code, 0)

        result = instructors[0].run("instructor grading assign-graders", ["pa1"])
        self.assertEquals(result.exit_code, 0)
        
        result = instructors[0].run("instructor grading list-grader-assignments", ["pa1"])
        self.assertEquals(result.exit_code, 0)
        
        result = instructors[0].run("instructor grading push-grading-branches", ["--to-staging", "pa1"])
        self.assertEquals(result.exit_code, 0)
        
        result = graders[0].run("grader create-local-grading-repos", [graders[0].user_id, "pa1"])
        self.assertEquals(result.exit_code, 0)        
                
        team1_grading_repo_path = ".chisubmit/repositories/%s/%s/%s" % (course_id, "pa1", teams[0])
        team2_grading_repo_path = ".chisubmit/repositories/%s/%s/%s" % (course_id, "pa1", teams[1])
            
        team_git_repos[0], team_git_paths[0] = graders[0].get_local_git_repository(team1_grading_repo_path)
        team_git_repos[1], team_git_paths[1] = graders[0].get_local_git_repository(team2_grading_repo_path)

        
        team1_rubric_path = "%s/pa1.rubric.txt" % team_git_paths[0] 
        team2_rubric_path = "%s/pa1.rubric.txt" % team_git_paths[1] 

        team1_rubric = """Points:
    The PA1 Tests:
        Points Possible: 50
        Points Obtained: 45

    The PA1 Design:
        Points Possible: 50
        Points Obtained: 30
        
Penalties:
    Used O(n^156) algorithm: -10
    Submitted code in a Word document: -30

Total Points: 35 / 100

Comments: >
    None"""

        with open(team1_rubric_path, "w") as f:
            f.write(team1_rubric)

        result = graders[0].run("grader validate-rubrics", [graders[0].user_id, "pa1", "--only", teams[0]])
        self.assertEquals(result.exit_code, 0)        
    
        team_git_repos[0].index.add(["pa1.rubric.txt"])
        team_git_repos[0].index.commit("Finished grading")
        
        
        
        with open("%s/bar" % team_git_paths[1], "a") as f:
            f.write("Great job!\n") 
            
        team2_rubric = """Points:
    The PA1 Tests:
        Points Possible: 50
        Points Obtained: 50

    The PA1 Design:
        Points Possible: 50
        Points Obtained: 45

Total Points: 95 / 100

Comments: >
    Great job!"""
                
        with open(team2_rubric_path, "w") as f:
            f.write(team2_rubric)

        result = graders[0].run("grader validate-rubrics", [graders[0].user_id, "pa1", "--only", teams[1]])
        self.assertEquals(result.exit_code, 0)        

        team_git_repos[1].index.add(["pa1.rubric.txt"])
        team_git_repos[1].index.add(["bar"])
        team_git_repos[1].index.commit("Finished grading")

        result = graders[0].run("grader validate-rubrics", [graders[0].user_id, "pa1"])
        self.assertEquals(result.exit_code, 0)                

        result = graders[0].run("grader push-grading-branches", [graders[0].user_id, "pa1"])
        self.assertEquals(result.exit_code, 0)                

        result = instructors[0].run("instructor grading pull-grading-branches", ["--from-staging", "pa1"])
        self.assertEquals(result.exit_code, 0)
        
        result = instructors[0].run("instructor grading collect-rubrics", ["pa1"])
        self.assertEquals(result.exit_code, 0)
        
        result = instructors[0].run("instructor grading list-grades")
        self.assertEquals(result.exit_code, 0)
                
        result = instructors[0].run("instructor grading push-grading-branches", ["--to-students", "pa1"])
        self.assertEquals(result.exit_code, 0)
        
    
        team_git_repos[0], team_git_paths[0] = students_team[0][0].get_local_git_repository(teams[0])
        team_git_repos[0].remote("origin").pull("pa1-grading:pa1-grading")
        team_git_repos[0].heads["pa1-grading"].checkout()        
        self.assertTrue(os.path.exists(team_git_paths[0] + "/pa1.rubric.txt"))

        team_git_repos[1], team_git_paths[1] = students_team[1][0].get_local_git_repository(teams[1])
        team_git_repos[1].remote("origin").pull("pa1-grading:pa1-grading")
        team_git_repos[1].heads["pa1-grading"].checkout()        
        self.assertTrue(os.path.exists(team_git_paths[0] + "/pa1.rubric.txt"))
        self.assertIn("Great job!", open(team_git_paths[1]+"/bar").read())
    
    
        



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import click

from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.cli.common import pass_course


@click.command(name="shell")
@click.pass_context  
@pass_course
def shell(ctx, course):
    try:
        from IPython import embed, __version__
        from IPython.config.loader import Config
        if __version__ < "1.1.0":
            print "You need IPython (>= 1.1.0) to run the chisubmit shell"
            ctx.exit(CHISUBMIT_FAIL)
    except ImportError:
        print "You need IPython (>= 1.1.0) to run the chisubmit shell"
        
    cfg = Config()
    cfg.TerminalInteractiveShell.banner1 = """
                      WELCOME TO THE CHISUBMIT SHELL
    
    Course: %s
    
    This is an IPython shell with the chisubmit data structures preloaded. 
    You can access the chisubmit objects through variable 'course'.
    
    CAREFUL: Most changes made through the shell will be propagated to the
             database. 
    
    """ % (course.name)
    
    prompt_config = cfg.PromptManager
    prompt_config.in_template = 'chisubmit> '
    prompt_config.in2_template = '   .\\D.> '
    prompt_config.out_template = '         > '
    embed(config = cfg)
        
    return CHISUBMIT_SUCCESS



from flask import url_for


class ExposedModel(object):

    @classmethod
    def uri(self):
        selfname = self.__name__.lower()
        # FIXME 23JULY14: assumptions unlikely to hold
        identifier = selfname + "_id"
        return url_for('get',
                       **{identifier: getattr(self, 'course_id'),
                          '_external': True})


class UniqueModel(object):

    def _unique(session, cls, hashfunc, queryfunc, constructor, arg, kw):
        cache = getattr(session, '_unique_cache', None)
        if cache is None:
            session._unique_cache = cache = {}

        key = (cls, hashfunc(*arg, **kw))
        if key in cache:
            return cache[key]
        else:
            with session.no_autoflush:
                q = session.query(cls)
                q = queryfunc(q, *arg, **kw)
                obj = q.first()
                if not obj:
                    obj = constructor(*arg, **kw)
                    session.add(obj)
            cache[key] = obj
            return obj

    @classmethod
    def as_unique(cls, session, *arg, **kw):
        return _unique(session,
                       cls, cls.unique_hash,
                       cls.unique_filter, cls,
                       arg, kw)




#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import os.path

import click

from chisubmit.common import CHISUBMIT_SUCCESS, ChisubmitException
import tempfile

def set_auth(server, config_profile):
    if config_profile.has_key("auth"):
        if config_profile["auth"]["type"] == "testing":
            server.set_auth_testing(config_profile["auth"]["password"])
        elif config_profile["auth"]["type"] == "ldap":
            server.set_auth_ldap(config_profile["auth"]["server-uri"],
                                 config_profile["auth"]["base-dn"])      
        else:
            raise ChisubmitException("Unrecognized authentication type: %s" % config_profile["auth"]["type"])



def get_server(config, profile):
    from chisubmit.backend.webapp.api import ChisubmitAPIServer
    
    p = config.get_server_profile(profile)

    if p is None:
        raise ChisubmitException("Specified server profile '%s' does not exist" % profile)
    
    server = ChisubmitAPIServer(p.get("debug", False))
    
    if p["db"] == "sqlite":
        dbfile = os.path.expanduser(p["db-sqlite-file"])
        server.connect_sqlite(dbfile)
    elif p["db"]  == "postgres":
        server.connect_postgres()
        
    set_auth(server, p)
                    
    return server

@click.command(name="start")
@click.option('--profile', type=str)
@click.option('--test-fixture', type=str)
@click.pass_context
def server_start(ctx, profile, test_fixture):
    config = ctx.obj["config"]
    
    if test_fixture is not None:
        from chisubmit.tests.fixtures import fixtures
        from chisubmit.tests.common import load_fixture
        from chisubmit.backend.webapp.api import ChisubmitAPIServer
        
        if not fixtures.has_key(test_fixture):
            raise ChisubmitException("Test fixture '%s' does not exist" % test_fixture)
        fixture, _ = fixtures[test_fixture]
        
        server = ChisubmitAPIServer(debug=True)
        
        if os.environ.get('WERKZEUG_RUN_MAIN') != "true":
            _ , db_filename = tempfile.mkstemp(prefix="chisubmit-test-")
            os.environ['CHISUBMIT_TEST_DB'] = db_filename
            server.connect_sqlite(db_filename)
            server.init_db()
            server.create_admin(api_key="admin")
            
            load_fixture(server.db, fixture)
            
            print "Fixture '%s' has been loaded on database %s" % (test_fixture, db_filename) 
        else:
            server.connect_sqlite(os.environ['CHISUBMIT_TEST_DB'])
            
        set_auth(server, config.get_server_profile(profile))
    else:
        server = get_server(config, profile)

    server.start()
    
    return CHISUBMIT_SUCCESS

@click.command(name="initdb")
@click.option('--profile', type=str)
@click.option('--admin-api-key', type=str)
@click.option('--force', is_flag=True)
@click.pass_context
def server_initdb(ctx, profile, admin_api_key, force):
    config = ctx.obj["config"]
    
    server = get_server(config, profile)

    server.init_db(drop_all = force)
    admin_key = server.create_admin(admin_api_key)
    
    if admin_key is not None:
        print "Chisubmit database has been created."
        print "The administrator's API key is %s" % admin_key
    else:
        print "The Chisubmit database already exists and it contains an 'admin' user"
        print "Use --force to re-create the database from scratch."
    
    return CHISUBMIT_SUCCESS





#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import click

from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.client.user import User
from chisubmit.client.course import Course
from chisubmit.common.utils import create_connection
from chisubmit.cli.shared.course import shared_course_list
import operator
import csv

@click.group(name="course")
@click.pass_context
def admin_course(ctx):
    pass


@click.command(name="add")
@click.argument('course_id', type=str)
@click.argument('name', type=str)
@click.pass_context
def admin_course_add(ctx, course_id, name):
    course = Course(id = course_id,
                    name = name)
    
    return CHISUBMIT_SUCCESS


@click.command(name="remove")
@click.argument('course_id', type=str)
@click.pass_context
def admin_course_remove(ctx, course_id):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    print "NOT IMPLEMENTED"

    return CHISUBMIT_SUCCESS


@click.command(name="show")
@click.argument('course_id', type=str)
@click.option("--include-users", is_flag=True)
@click.option("--include-assignments", is_flag=True)
@click.pass_context
def admin_course_show(ctx, course_id, include_users, include_assignments):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    print course.id, course.name
    if len(course.options) == 0:
        print "No options"
    else:
        print "Options"
        for name, value in course.options.items():
            print "  %s: %s" % (name, value)
    print

    if include_users:
        print "INSTRUCTORS"
        print "-----------"
        for i in course.instructors:
            print "%s: %s, %s <%s>" % (i.user.id, i.user.last_name, i.user.first_name, i.user.email)
        print
            
        print "GRADERS"
        print "-------"
        for g in course.graders:
            print "%s: %s, %s <%s>" % (g.user.id, g.user.last_name, g.user.first_name, g.user.email)
        print
        
        print "STUDENTS"
        print "--------"
        for s in course.students:
            print "%s: %s, %s <%s>" % (s.user.id, s.user.last_name, s.user.first_name, s.user.email)
        print

    if include_assignments:
        print "ASSIGNMENTS"
        print "-----------"
        for a in course.assignments:
            print "%s: %s (Due: %s)" % (a.id, a.name, a.deadline)
        print

    
    return CHISUBMIT_SUCCESS




@click.command(name="add-instructor")
@click.argument('course_id', type=str)
@click.argument('user_id', type=str)
@click.pass_context
def admin_course_add_instructor(ctx, course_id, user_id):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    user = User.from_id(user_id)
    if course is None:
        print "User %s does not exist" % user_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    course.add_instructor(user)
    
@click.command(name="add-grader")
@click.argument('course_id', type=str)
@click.argument('user_id', type=str)
@click.pass_context
def admin_course_add_grader(ctx, course_id, user_id):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    user = User.from_id(user_id)
    if course is None:
        print "User %s does not exist" % user_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    course.add_grader(user)
    
@click.command(name="add-student")
@click.argument('course_id', type=str)
@click.argument('user_id', type=str)
@click.pass_context
def admin_course_add_student(ctx, course_id, user_id):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    user = User.from_id(user_id)
    if course is None:
        print "User %s does not exist" % user_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    course.add_student(user)  
    
    
@click.command(name="load-students")
@click.argument('course_id', type=str)
@click.argument('csv_file', type=click.File('rb'))
@click.argument('csv_userid_column', type=str)
@click.argument('csv_fname_column', type=str)
@click.argument('csv_lname_column', type=str)
@click.argument('csv_email_column', type=str)
@click.option('--dry-run', is_flag=True)
@click.option('--id-from-email', is_flag=True)
@click.pass_context
def admin_course_load_students(ctx, course_id, csv_file, csv_userid_column, csv_fname_column, csv_lname_column, csv_email_column, dry_run, id_from_email):   
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)   
                
    csvf = csv.DictReader(csv_file)
            
    for col in (csv_userid_column, csv_fname_column, csv_lname_column, csv_email_column):
        if col not in csvf.fieldnames:
            print "CSV file %s does not have a '%s' column" % (csv_file, col)
            ctx.exit(CHISUBMIT_FAIL)
        
    for entry in csvf:
        user_id = entry[csv_userid_column]
        if id_from_email:
            user_id = user_id.split("@")[0].strip()
        
        first_name = entry[csv_fname_column]
        last_name = entry[csv_lname_column]
        email = entry[csv_email_column]

        print "Processing %s (%s, %s)" % (user_id, last_name, first_name)

        user = User.from_id(user_id)
    
        if user is not None:
            print "- User %s already exists." % user_id
        else:
            print "- Creating user %s" % user_id
            if not dry_run:
                user = User(id = user_id,
                            first_name = first_name,
                            last_name = last_name,
                            email = email)
        
        student = course.get_student(user_id)
        if student is not None:
            print "- User %s is already a student in %s" % (user_id, course_id)
        else:
            print "- Adding %s to %s" % (user_id, course_id)
            if not dry_run:
                course.add_student(user)
        
        print 
    
@click.command(name="set-option")
@click.argument('course_id', type=str)
@click.argument('option_name', type=str)
@click.argument('option_value', type=str)
@click.pass_context
def admin_course_set_option(ctx, course_id, option_name, option_value):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    course.set_option(option_name, option_value)


@click.command(name="setup-repo")
@click.argument('course_id', type=str)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_setup_repo(ctx, course_id, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)

    conn.init_course(course)

    return CHISUBMIT_SUCCESS

@click.command(name="unsetup-repo")
@click.argument('course_id', type=str)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_unsetup_repo(ctx, course_id, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)

    conn.deinit_course(course)

    return CHISUBMIT_SUCCESS

@click.command(name="update-repo-access")
@click.argument('course_id', type=str)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_update_repo_access(ctx, course_id, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    
    
    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)
    
    conn.update_instructors(course)
    conn.update_graders(course)


@click.command(name="create-repos")
@click.argument('course_id', type=str)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_create_repos(ctx, course_id, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    teams = course.get_teams()

    if len(teams) == 0:
        print "Course %s has no teams. No repositories to create." % course_id
        ctx.exit(CHISUBMIT_FAIL)  

    max_len = max([len(t.id) for t in teams])

    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)
    
    v = ctx.obj["verbose"]
    already_has_repository = 0
    warning = 0
    created = 0
    for team in sorted(teams, key=operator.attrgetter("id")):
        if not team.is_complete():
            warning += 1
            print "%-*s  WARNING. Team registration is incomplete." % (max_len, team.id)
            continue
        
        if conn.exists_team_repository(course, team):
            already_has_repository += 1
            if v: print "%-*s  SKIPPING. Already has a repository." % (max_len, team.id)
            continue

        if not staging:        
            students = [s for s in course.students if s.user.id in [ts.user.id for ts in team.students]]
                
            missing = []
            for s in students:
                if not hasattr(s, "repo_info") or s.repo_info is None or not s.repo_info.has_key("git-username"):
                    missing.append(s.user.id)
                    
            if len(missing) > 0:
                warning += 1
                print "%-20s WARNING. These students haven't set their git usernames: %s" % (team.id, ",".join(missing))
                continue
        
        try:
            conn.create_team_repository(course, team)
            created += 1
            print "%-20s CREATED" % team.id
        except Exception, e:
            print "%-20s Unexpected exception %s: %s" % (team.id, e.__class__.__name__, e.message)

    print
    print "Existing: %i" % already_has_repository
    print "Created : %i" % created
    print "Warnings: %i" % warning

    return CHISUBMIT_SUCCESS


@click.command(name="team-repo-create")
@click.argument('course_id', type=str)
@click.argument('team_id', type=str)
@click.option('--ignore-existing', is_flag=True)
@click.option('--public', is_flag=True)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_team_repo_create(ctx, course_id, team_id, ignore_existing, public, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    team = course.get_team(team_id)
    if team is None:
        print "Team %s does not exist" % team_id
        ctx.exit(CHISUBMIT_FAIL)

    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)
    
    if conn.exists_team_repository(course, team) and not ignore_existing:
        print "Team %s already has a repository" % team_id
        ctx.exit(CHISUBMIT_FAIL)
    else:
        conn.create_team_repository(course, team, fail_if_exists = not ignore_existing, private = not public)

    return CHISUBMIT_SUCCESS


@click.command(name="team-repo-update")
@click.argument('course_id', type=str)
@click.argument('team_id', type=str)
@click.pass_context
def admin_course_team_repo_update(ctx, course_id, team_id):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    team = course.get_team(team_id)
    if team is None:
        print "Team %s does not exist" % team_id
        ctx.exit(CHISUBMIT_FAIL)

    #if team.github_repo is None:
    #    print "Team %s does not have a repository." % team.id
    #    return CHISUBMIT_FAIL

    conn = create_connection(course, ctx.obj['config'], staging = False)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)

    conn.update_team_repository(course, team)
    return CHISUBMIT_SUCCESS


@click.command(name="team-repo-remove")
@click.argument('course_id', type=str)
@click.argument('team_id', type=str)
@click.option('--ignore-non-existing', is_flag=True)
@click.option('--staging', is_flag=True)
@click.pass_context
def admin_course_team_repo_remove(ctx, course_id, team_id, ignore_non_existing, staging):
    course = Course.from_id(course_id)
    if course is None:
        print "Course %s does not exist" % course_id
        ctx.exit(CHISUBMIT_FAIL)    

    team = course.get_team(team_id)
    if team is None:
        print "Team %s does not exist" % team_id
        ctx.exit(CHISUBMIT_FAIL)

    conn = create_connection(course, ctx.obj['config'], staging)
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)

    if not conn.exists_team_repository(course, team) and not ignore_non_existing:
        print "WARNING: Cannot delete repository because it doesn't exist"
        ctx.exit(CHISUBMIT_FAIL)
    else:
        conn.delete_team_repository(course, team, fail_if_not_exists = not ignore_non_existing)

    return CHISUBMIT_SUCCESS


admin_course.add_command(shared_course_list)


admin_course.add_command(admin_course_add)
admin_course.add_command(admin_course_remove)
admin_course.add_command(admin_course_show)
admin_course.add_command(admin_course_add_instructor)
admin_course.add_command(admin_course_add_grader)
admin_course.add_command(admin_course_add_student)
admin_course.add_command(admin_course_load_students)
admin_course.add_command(admin_course_set_option)
admin_course.add_command(admin_course_setup_repo)
admin_course.add_command(admin_course_unsetup_repo)
admin_course.add_command(admin_course_update_repo_access)
admin_course.add_command(admin_course_create_repos)
admin_course.add_command(admin_course_team_repo_create)
admin_course.add_command(admin_course_team_repo_update)
admin_course.add_command(admin_course_team_repo_remove)





#!/usr/bin/python

import sys
import traceback

from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage as CredentialStorage
from oauth2client.tools import run as run_oauth2

import gdata.spreadsheets.client

import chisubmit.core
from chisubmit.core.model import Course, Team, Student
from chisubmit.core.repos import GithubConnection
from chisubmit.core import ChisubmitException


CLIENT_SECRETS_FILE = 'client_secrets.json'
CREDENTIALS_FILE = 'credentials.json'
SCOPE = "https://spreadsheets.google.com/feeds"
SPREADSHEET_KEY = open("spreadsheet_key").read().strip()

GROUPNAME = 'groupname'
STUDENT1_CNETID = 'cnetid1'
STUDENT1_FNAME = 'firstname1'
STUDENT1_LNAME = 'lastname1'
STUDENT1_EMAIL = 'e-mail1'
STUDENT1_GITHUB = 'githubusername1'
STUDENT2_CNETID = 'cnetid2'
STUDENT2_FNAME = 'firstname2'
STUDENT2_LNAME = 'lastname2'
STUDENT2_EMAIL = 'e-mail2'
STUDENT2_GITHUB = 'githubusername2'


if len(sys.argv) != 2:
    print "Usage: create_groups.py COURSE_ID"
    exit(1)

course_id = sys.argv[1]

# Initialize chisubmit
# We assume default chisubmit directory and configuration file are used
chisubmit.core.init_chisubmit()
course = Course.from_course_id(course_id)

if course is None:
    print "Course %s does not exist"
    exit(1)

# Do the OAuth2 dance
flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=SCOPE, message="Client secrets file does not exist.")
credential_storage = CredentialStorage(CREDENTIALS_FILE)
credentials = credential_storage.get()

if credentials == None or credentials.invalid:
    credentials = run_oauth2(flow, credential_storage)

client = gdata.spreadsheets.client.SpreadsheetsClient()
token = gdata.gauth.OAuth2Token(client_id=flow.client_id, 
                                client_secret=flow.client_secret, 
                                scope=flow.scope,
                                user_agent="chisubmit", 
                                access_token=credentials.access_token, 
                                refresh_token=credentials.refresh_token)
token.authorize(client)

# Open spreadsheet
worksheets = client.get_worksheets(SPREADSHEET_KEY)

# Get first worksheet
worksheet_id = worksheets.entry[0].get_worksheet_id()

feed = client.get_list_feed(SPREADSHEET_KEY, worksheet_id)

for row in feed.entry:
    print "============================================"
    group = row.to_dict()

    if course.teams.has_key(group[GROUPNAME]):
        print "Skipping '%s'. Already exists." % group[GROUPNAME] 
        continue
    
    print "Adding group '%s'" % group[GROUPNAME]
    
    if course.students.has_key(group[STUDENT1_CNETID]):
        print "Skipping %s %s (%s). Already exists." % (group[STUDENT1_FNAME], group[STUDENT1_LNAME], group[STUDENT1_CNETID])
        student1 = course.students[group[STUDENT1_CNETID]];
    else:    
        student1 = Student(student_id = group[STUDENT1_CNETID],
                          first_name = group[STUDENT1_FNAME],
                          last_name  = group[STUDENT1_LNAME],
                          email      = group[STUDENT1_EMAIL],
                          github_id  = group[STUDENT1_GITHUB])
        course.add_student(student1)
        print "Added student %s %s (%s)." % (student1.first_name, student1.last_name, student1.id)

    if course.students.has_key(group[STUDENT2_CNETID]):
        print "Skipping %s %s (%s). Already exists." % (group[STUDENT2_FNAME], group[STUDENT2_LNAME], group[STUDENT2_CNETID])
        student2 = course.students[group[STUDENT2_CNETID]];
    else:    
        student2 = Student(student_id = group[STUDENT2_CNETID],
                          first_name = group[STUDENT2_FNAME],
                          last_name  = group[STUDENT2_LNAME],
                          email      = group[STUDENT2_EMAIL],
                          github_id  = group[STUDENT2_GITHUB])
        course.add_student(student2)
        print "Added student %s %s (%s)." % (student2.first_name, student2.last_name, student2.id)
    
    team = Team(team_id = group[GROUPNAME])
    course.add_team(team)
    team.add_student(student1)     
    team.add_student(student2)     
    print "Added team '%s'" % (team.id)
    
    course.save()
    
    print "Creating GitHub repository...",
    try:
        github_access_token = chisubmit.core.get_github_token()
        gh = GithubConnection(github_access_token, course.github_organization)
        gh.create_team_repository(course, team, True)
    except ChisubmitException, ce:
        print "ERROR: %s" % ce.message
        ce.print_exception()
        continue
    except Exception, e:
        print "ERROR: Unexpected exception"
        print traceback.format_exc()
        continue    
    print "done"

    course.save()

print "============================================"





import click

from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.repos.grading import GradingGitRepo
from chisubmit.rubric import RubricFile, ChisubmitRubricException
from chisubmit.cli.common import create_grading_repos, get_teams,\
    gradingrepo_push_grading_branch, gradingrepo_pull_grading_branch
from chisubmit.cli.common import pass_course
from chisubmit.common.utils import create_connection

import csv
import operator
import random
import itertools
import os.path
from pprint import pprint

@click.group(name="grading")
@click.pass_context
def instructor_grading(ctx):
    pass


@click.command(name="set-grade")
@click.argument('team_id', type=str)
@click.argument('assignment_id', type=str)
@click.argument('grade_component_id', type=str)
@click.argument('points', type=float)
@pass_course
@click.pass_context
def instructor_grading_set_grade(ctx, course, team_id, assignment_id, grade_component_id, points):   
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    team = course.get_team(team_id)
    if team is None:
        print "Team %s does not exist" % team_id
        ctx.exit(CHISUBMIT_FAIL)
        
    ta = team.get_assignment(assignment_id)
    if ta is None:
        print "Team %s is not registered for assignment %s" % (team_id, assignment_id)
        ctx.exit(CHISUBMIT_FAIL)

    grade_component = assignment.get_grade_component(grade_component_id)
    if not grade_component:
        print "Assignment %s does not have a grade component '%s'" % (assignment.id, grade_component)
        ctx.exit(CHISUBMIT_FAIL)
        
    if points < 0 or points > grade_component.points:
        print "Invalid grade value %.2f (%s must be 0 <= x <= %.2f)" % (points, grade_component_id, grade_component.points)
        ctx.exit(CHISUBMIT_FAIL)
    
    team.set_assignment_grade(assignment_id, grade_component.id, points)


@click.command(name="load-grades")
@click.argument('assignment_id', type=str)
@click.argument('grade_component_id', type=str)
@click.argument('csv_file', type=click.File('rb'))
@click.argument('csv_team_column', type=str)
@click.argument('csv_grade_column', type=str)
@pass_course
@click.pass_context
def instructor_grading_load_grades(ctx, course, assignment_id, grade_component_id, csv_file, csv_team_column, csv_grade_column):   
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    grade_component = assignment.get_grade_component(grade_component_id)
    if not grade_component:
        print "Assignment %s does not have a grade component '%s'" % (assignment.id, grade_component)
        ctx.exit(CHISUBMIT_FAIL)
        
    csvf = csv.DictReader(csv_file)
            
    if csv_team_column not in csvf.fieldnames:
        print "CSV file %s does not have a '%s' column" % (csv_file, csv_team_column)
        ctx.exit(CHISUBMIT_FAIL)
        
    if csv_grade_column not in csvf.fieldnames:
        print "CSV file %s does not have a '%s' column" % (csv_file, csv_grade_column)
        ctx.exit(CHISUBMIT_FAIL)
            
    for entry in csvf:
        team_id = entry[csv_team_column]
        
        team = course.get_team(team_id)
        if team is None:
            print "%-40s SKIPPING. Not a team in course %s" % (team_id, course.id)
            continue

        ta = team.get_assignment(assignment_id)
        if ta is None:
            print "%-40s SKIPPING. Not registered for assignment %s" % (team_id, assignment_id)
            continue
        
        if ta.submitted_at is None:
            print "%-40s SKIPPING. Has not submitted assignment %s yet" % (team_id, assignment_id)
            continue
    
        grade = entry[csv_grade_column]
    
        if grade is None or grade == "":
            grade = 0
        else:
            grade = float(grade)

        if grade < 0 or grade > grade_component.points:
            print "%-40s SKIPPING. Invalid grade value %.2f (%s must be 0 <= x <= %.2f)" % (team_id, grade, grade_component_id, grade_component.points)
            continue
        
        try:
            team.set_assignment_grade(assignment_id, grade_component.id, grade)
            print "%-40s %s <- %.2f" % (team_id, grade_component_id, grade)
        except Exception:
            raise
            

@click.command(name="add-conflict")
@click.argument('grader_id', type=str)
@click.argument('student_id', type=str)
@pass_course
@click.pass_context
def instructor_grading_add_conflict(ctx, course, grader_id, student_id):
    grader = course.get_grader(grader_id)
    if not grader:
        print "Grader %s does not exist" % grader_id
        ctx.exit(CHISUBMIT_FAIL)

    student = course.get_student(student_id)
    if not student:
        print "Student %s does not exist" % student_id
        ctx.exit(CHISUBMIT_FAIL)

    if student in grader.conflicts:
        print "Student %s is already listed as a conflict for grader %s" % (student.id, grader.id)

    grader.conflicts.append(student)

    return CHISUBMIT_SUCCESS

@click.command(name="list-grades")
@pass_course
@click.pass_context
def instructor_grading_list_grades(ctx, course):
    students = [s for s in course.students if not s.dropped]
    assignments = course.assignments

    students.sort(key=operator.attrgetter("user.last_name"))
    assignments.sort(key=operator.attrgetter("deadline"))

    student_grades = dict([(s.user.id,dict([(a.id,{}) for a in assignments])) for s in students])

    for team in course.teams:
        for team_assignment in team.assignments:
            assignment_id = team_assignment.assignment_id
            for student in team.students:
                student_id = student.user.id
                if student_grades.has_key(student_id):
                    g = student_grades[student_id][assignment_id]
                    for grade in team_assignment.grades:
                        gc_id = grade.grade_component_id
                        if g.has_key(gc_id):
                            print "Warning: %s already has a grade for grade component %i" % (student_id, gc_id)
                        else:
                            g[gc_id] = grade.points
                    g["__PENALTIES"] = team_assignment.get_total_penalties()
                    g["__TOTAL"] = team_assignment.get_total_grade()

    fields = ["Last Name","First Name"]
    for assignment in assignments:
        fields += ["%s - %s" % (assignment.id, gc.description) for gc in assignment.grade_components]
        fields.append("%s - Penalties" % assignment.id)
        fields.append("%s - Total" % assignment.id)

    print ",".join(fields)

    for student in students:
        fields = [student.user.last_name, student.user.first_name]
        for assignment in assignments:
            grades = student_grades[student.user.id][assignment.id]
            for gc in assignment.grade_components:
                if not grades.has_key(gc.id):
                    fields.append("")
                else:
                    fields.append(str(grades[gc.id]))
            if len(grades) == 0:
                fields += ["",""]
            else:
                fields.append(str(grades["__PENALTIES"]))
                fields.append(str(grades["__TOTAL"]))

        print ",".join(fields)


@click.command(name="assign-graders")
@click.argument('assignment_id', type=str)
@click.option('--from-assignment', type=str)
@click.option('--avoid-assignment', type=str)
@click.option('--only-graders', type=str)
@click.option('--reset', is_flag=True)
@pass_course
@click.pass_context
def instructor_grading_assign_graders(ctx, course, assignment_id, from_assignment, avoid_assignment, only_graders, reset):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    from_assignment = None
    if from_assignment is not None:
        from_assignment = course.get_assignment(from_assignment)
        if from_assignment is None:
            print "Project %s does not exist" % from_assignment
            ctx.exit(CHISUBMIT_FAIL)

    avoid_assignment = None
    if avoid_assignment is not None:
        avoid_assignment = course.get_assignment(avoid_assignment)
        if avoid_assignment is None:
            print "Project %s does not exist" % avoid_assignment
            ctx.exit(CHISUBMIT_FAIL)

    if reset and from_assignment is not None:
        print "--reset and --from_assignment are mutually exclusive"
        ctx.exit(CHISUBMIT_FAIL)

    if avoid_assignment is not None and from_assignment is not None:
        print "--avoid_assignment and --from_assignment are mutually exclusive"
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment)
    graders = course.graders[:]
    
    if only_graders is None:
        graders = course.graders[:]
    else:
        gd = {g.user.id: g for g in course.graders}
        only_graders = only_graders.strip().split(",")
        
        graders = []
        for g in only_graders:
            if g in gd:
                graders.append(gd[g])
            else:
                print "No such grader: %s" % g
                ctx.exit(CHISUBMIT_FAIL)        
    
    if len(graders) == 0:
        print "There are ZERO graders in this course!"
        ctx.exit(CHISUBMIT_FAIL)

    min_teams_per_grader = len(teams) / len(graders)
    extra_teams = len(teams) % len(graders)

    teams_per_grader = dict([(g.user.id, min_teams_per_grader) for g in graders])
    teams_per_grader_assigned = dict([(g.user.id, 0) for g in graders])
    
    random.seed(course.id + assignment_id)
    random.shuffle(graders)

    # so many graders in this course that some will end up expecting zero
    # teams to grade. Make sure they are able to get at least one.
    for g in graders[:extra_teams]:
        teams_per_grader[g.user.id] += 1

    team_grader = {t.id: None for t in teams}
    ta = {t.id: t.get_assignment(assignment.id) for t in teams}

    if from_assignment is not None:
        common_teams = [t for t in course.teams if t.has_assignment(assignment.id) and t.has_assignment(from_assignment.id)]
        for t in common_teams:
            team_assignment_from = t.get_assignment(from_assignment.id)

            # try to assign the same grader that would grade the same team's other assignment
            grader = team_assignment_from.grader
            if grader is not None and teams_per_grader[grader.user.id] > 0:
                team_grader[t.id] = grader.user.id 
                teams_per_grader[grader.user.id] -= 1
                teams_per_grader_assigned[grader.user.id] += 1

    if not reset:
        for t in teams:
            if ta[t.id].grader is None:
                team_grader[t.id] = None
            else:
                grader_id = ta[t.id].grader.id 
                team_grader[t.id] = grader_id
                teams_per_grader[grader_id] -= 1
                teams_per_grader_assigned[grader_id] += 1

    not_ready_for_grading = []
    ta_avoid = {}
    graders_cycle = itertools.cycle(graders)
    for t in teams:
        if team_grader[t.id] is not None:
            continue
        
        if not t.has_assignment_ready_for_grading(assignment):
            not_ready_for_grading.append(t.id)
            continue

        for g in graders_cycle:
            if teams_per_grader[g.user.id] == 0:
                continue
            else:
                if avoid_assignment is not None:
                    taa = ta_avoid.setdefault(t.id, t.get_assignment(avoid_assignment.id))
                    if taa.grader.user.id == grader.user.id:
                        continue
                
                valid = True
                for s in t.students:
                    conflicts = g.get_conflicts()
                    if s.user.id in conflicts:
                        valid = False
                        break

                if valid:
                    team_grader[t.id] = g.user.id 
                    teams_per_grader[g.user.id] -= 1
                    teams_per_grader_assigned[g.user.id] += 1                        
                    break                    

    for t in teams:
        if team_grader[t.id] is None:
            if t.id not in not_ready_for_grading:
                print "Team %s has no grader" % (t.id)
            else:
                print "Team %s's submission isn't ready for grading yet" % (t.id)
        else:
            if ta[t.id].grader !=  team_grader[t.id]:
                t.set_assignment_grader(assignment.id, team_grader[t.id])

    print 
    for grader_id, assigned in teams_per_grader_assigned.items():
        if teams_per_grader[grader_id] != 0:
            print grader_id, assigned, "(still needs to be assigned %i more assignments)" % (teams_per_grader[grader_id])
        else:
            print grader_id, assigned

    return CHISUBMIT_SUCCESS


@click.command(name="list-grader-assignments")
@click.argument('assignment_id', type=str)
@click.option('--grader-id', type=str)
@pass_course
@click.pass_context
def instructor_grading_list_grader_assignments(ctx, course, assignment_id, grader_id):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    if grader_id is not None:
        grader = course.get_grader(grader_id)
        if grader is None:
            print "Grader %s does not exist" % grader_id
            ctx.exit(CHISUBMIT_FAIL)
    else:
        grader = None

    teams = get_teams(course, assignment)
    teams.sort(key=operator.attrgetter("id"))

    for t in teams:
        team_assignment = t.get_assignment(assignment.id)
        if grader is None:
            if team_assignment.grader is None:
                grader_str = "<no-grader-assigned>"
            else:
                grader_str = team_assignment.grader.id
            print t.id, grader_str
        else:
            if grader == team_assignment.grader:
                print t.id

    return CHISUBMIT_SUCCESS


@click.command(name="list-submissions")
@click.argument('assignment_id', type=str)
@pass_course
@click.pass_context
def instructor_grading_list_submissions(ctx, course, assignment_id):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment)
    teams.sort(key=operator.attrgetter("id"))

    conn = create_connection(course, ctx.obj['config'])
    
    if conn is None:
        print "Could not connect to git server."
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        ta = team.get_assignment(assignment.id)

        if ta.submitted_at is None:
            print "%25s NOT SUBMITTED" % team.id
        else:
            submission_commit = conn.get_commit(course, team, ta.commit_sha)
            if submission_commit is not None:
                if team.has_assignment_ready_for_grading(assignment):
                    print "%25s SUBMITTED (READY for grading)" % team.id
                else:
                    print "%25s SUBMITTED (NOT READY for grading)" % team.id
            else:
                print "%25s ERROR: Submitted but commit %s not found in repository" % (team.id, ta.commit_sha)
    
    return CHISUBMIT_SUCCESS


@click.command(name="show-grading-status")
@click.argument('assignment_id', type=str)
@click.option('--by-grader', is_flag=True)
@pass_course
@click.pass_context
def instructor_grading_show_grading_status(ctx, course, assignment_id, by_grader):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment)
    teams.sort(key=operator.attrgetter("id"))
    
    team_status = []
    graders = set()
    
    for team in teams:
        ta = team.get_assignment(assignment_id)
        grade_ids = [g.grade_component_id for g in ta.grades]

        if ta.grader is not None:
            grader_id = ta.grader.id
        else:
            grader_id = "<no grader assigned>"
            
        graders.add(grader_id)

        has_some = False
        has_all = True
        for gc in assignment.grade_components:
            if gc.id in grade_ids:
                has_some = True
            else:
                has_all = False

        if not has_some:
            team_status.append((team.id, grader_id, "NOT GRADED"))
        elif has_all:
            team_status.append((team.id, grader_id, "GRADED"))
        else:
            team_status.append((team.id, grader_id, "PARTIALLY GRADED"))

    if not by_grader:
        for team, grader, status in team_status:
            print "%-40s %-20s %s" % (team, status, grader)
    else:
        for grader in sorted(list(graders)):
            print grader
            print "-" * len(grader)
            
            team_status_grader = [ts for ts in team_status if ts[1] == grader]
            
            for team, _, status in team_status_grader:
                print "%-40s %s" % (team, status)

            print

    return CHISUBMIT_SUCCESS

@click.command(name="create-grading-repos")
@click.argument('assignment_id', type=str)
@click.option('--all-teams', is_flag=True)
@pass_course
@click.pass_context
def instructor_grading_create_grading_repos(ctx, course, assignment_id, all_teams):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, only_ready_for_grading=not all_teams)

    repos = create_grading_repos(ctx.obj['config'], course, assignment, teams)

    if repos == None:
        ctx.exit(CHISUBMIT_FAIL)

    return CHISUBMIT_SUCCESS


@click.command(name="create-grading-branches")
@click.argument('assignment_id', type=str)
@click.option('--all-teams', is_flag=True)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def instructor_grading_create_grading_branches(ctx, course, assignment_id, all_teams, only):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, only = only, only_ready_for_grading=not all_teams)

    if teams is None:
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        repo = GradingGitRepo.get_grading_repo(ctx.obj['config'], course, team, assignment)

        if repo is None:
            print "%s does not have a grading repository" % team.id
            continue
        
        ta = team.get_assignment(assignment.id)

        if ta.submitted_at is None:
            print "Skipping grading branch. %s has not submitted." % team.id
        elif repo.has_grading_branch():
            print "Skipping grading branch. %s already has a grading branch." % team.id
        else:
            repo.create_grading_branch()
            print "Created grading branch for %s" % team.id

    return CHISUBMIT_SUCCESS


@click.command(name="push-grading-branches")
@click.argument('assignment_id', type=str)
@click.option('--to-staging', is_flag=True)
@click.option('--to-students', is_flag=True)
@click.option('--all-teams', is_flag=True)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def instructor_grading_push_grading_branches(ctx, course, assignment_id, to_staging, to_students, all_teams, only):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, only = only, only_ready_for_grading=not all_teams)

    if teams is None:
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        print ("Pushing grading branch for team %s... " % team.id),
        gradingrepo_push_grading_branch(ctx.obj['config'], course, team, assignment, to_staging = to_staging, to_students = to_students)
        print "done."

    return CHISUBMIT_SUCCESS



@click.command(name="pull-grading-branches")
@click.argument('assignment_id', type=str)
@click.option('--from-staging', is_flag=True)
@click.option('--from-students', is_flag=True)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def instructor_grading_pull_grading_branches(ctx, course, assignment_id, from_staging, from_students, only):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, only = only)

    if teams is None:
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        print "Pulling grading branch for team %s... " % team.id
        gradingrepo_pull_grading_branch(ctx.obj['config'], course, team, assignment, from_staging = from_staging, from_students = from_students)

    return CHISUBMIT_SUCCESS



@click.command(name="add-rubrics")
@click.argument('assignment_id', type=str)
@click.option('--commit', is_flag=True)
@click.option('--all-teams', is_flag=True)
@pass_course
@click.pass_context
def instructor_grading_add_rubrics(ctx, course, assignment_id, commit, all_teams):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, only_ready_for_grading=not all_teams)

    for team in teams:
        repo = GradingGitRepo.get_grading_repo(ctx.obj['config'], course, team, assignment)
        team_assignment = team.get_assignment(assignment_id)
        rubric = RubricFile.from_assignment(assignment, team_assignment)
        rubricfile = "%s.rubric.txt" % assignment.id
        rubricfilepath = "%s/%s" % (repo.repo_path, rubricfile)
        
        if commit:
            if not os.path.exists(rubricfilepath):
                rubric.save(rubricfilepath, include_blank_comments=True)
                rv = repo.commit([rubricfile], "Added grading rubric")
                if rv:
                    print rubricfilepath, "(COMMITTED)"
                else:
                    print rubricfilepath, "(NO CHANGES - Not committed)"
            else:
                print rubricfilepath, "(SKIPPED - already exists)"
        else:
            rubric.save(rubricfilepath, include_blank_comments=True)
            print rubricfilepath


@click.command(name="collect-rubrics")
@click.argument('assignment_id', type=str)
@click.option('--dry-run', is_flag=True)
@click.option('--grader-id', type=str)
@pass_course
@click.pass_context
def instructor_grading_collect_rubrics(ctx, course, assignment_id, dry_run, grader_id):
    assignment = course.get_assignment(assignment_id)
    if assignment is None:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    if grader_id is not None:
        grader = course.get_grader(grader_id)
        if grader is None:
            print "Grader %s does not exist" % grader_id
            ctx.exit(CHISUBMIT_FAIL)
    else:
        grader = None

    gcs = assignment.grade_components

    teams = get_teams(course, assignment, grader=grader)

    for team in teams:
        repo = GradingGitRepo.get_grading_repo(ctx.obj['config'], course, team, assignment)
        if repo is None:
            print "Repository for %s does not exist" % (team.id)
            continue

        rubricfile = repo.repo_path + "/%s.rubric.txt" % assignment.id

        if not os.path.exists(rubricfile):
            print "Repository for %s does not have a rubric for assignment %s" % (team.id, assignment.id)
            continue

        try:
            rubric = RubricFile.from_file(open(rubricfile), assignment)
        except ChisubmitRubricException, cre:
            print "ERROR: Rubric for %s does not validate (%s)" % (team.id, cre.message)
            continue

        points = []
        for gc in gcs:
            grade = rubric.points[gc.description]
            if grade is None:
                points.append(0.0)
            else:
                if not dry_run:
                    team.set_assignment_grade(assignment.id, gc.id, grade)
                points.append(grade)

        penalties = {}
        total_penalties = 0.0
        if rubric.penalties is not None:
            for desc, p in rubric.penalties.items():
                penalties[desc] = p
                total_penalties += p

        if not dry_run:
            team.set_assignment_penalties(assignment.id, penalties)

        if ctx.obj["verbose"]:
            print team.id
            print "+ %s" % points
            print "- %.2f" % total_penalties

        if not dry_run:
            new_team = course.get_team(team.id)
            assignment_team = new_team.get_assignment(assignment.id)
            total_grade = assignment_team.get_total_grade()
        else:
            total_grade = sum(points) + total_penalties
            
        if ctx.obj["verbose"]:
            print "TOTAL: %.2f" % total_grade
            print
        else:
            print "%-40s %.2f" % (team.id, total_grade)
            

instructor_grading.add_command(instructor_grading_set_grade)
instructor_grading.add_command(instructor_grading_load_grades)
instructor_grading.add_command(instructor_grading_list_grades)
instructor_grading.add_command(instructor_grading_assign_graders)
instructor_grading.add_command(instructor_grading_list_grader_assignments)
instructor_grading.add_command(instructor_grading_list_submissions)
instructor_grading.add_command(instructor_grading_show_grading_status)
instructor_grading.add_command(instructor_grading_create_grading_repos)
instructor_grading.add_command(instructor_grading_create_grading_branches)
instructor_grading.add_command(instructor_grading_push_grading_branches)
instructor_grading.add_command(instructor_grading_pull_grading_branches)
instructor_grading.add_command(instructor_grading_add_rubrics)
instructor_grading.add_command(instructor_grading_collect_rubrics)



from . import test_admin_courses
from unittest.suite import TestSuite
from chisubmit.tests.cli import test_auth

test_cases = [test_admin_courses, test_auth]

def load_tests(loader, tests, pattern):
    suite = TestSuite()
    for test_module in test_cases:
        tests = loader.loadTestsFromModule(test_module)
        suite.addTests(tests)
    return suite


from chisubmit.backend.webapp.auth import Auth


class TestingAuth(Auth):
    
    def __init__(self, server, password):
        super(TestingAuth, self).__init__(server)

        self.password = password
        
        
    def check_auth(self, username, password):
        print "CHECK", username, password
        if password == self.password:
            return True
        else:
            return False


import click
from chisubmit.cli.admin.course import admin_course
from chisubmit.cli.admin.user import admin_user

@click.group()
@click.pass_context
def admin(ctx):
    pass

admin.add_command(admin_user)
admin.add_command(admin_course)



import git

from chisubmit.repos import RemoteRepositoryConnectionBase, GitCommit
from chisubmit.common import ChisubmitException
import os
from gitdb.exc import BadObject
import datetime
from chisubmit.repos.local import LocalGitRepo

class TestingConnection(RemoteRepositoryConnectionBase):

    VALID_CREDENTIALS = "testing-credentials"

    def __init__(self, connection_string, staging):
        RemoteRepositoryConnectionBase.__init__(self, connection_string, staging)
        
    @staticmethod
    def get_server_type_name():
        return "Testing"
    
    @staticmethod
    def get_connstr_mandatory_params():
        return ["local_path"]

    @staticmethod
    def get_connstr_optional_params():
        return []
    
    @staticmethod
    def get_credentials(username, password, delete_repo = False):
        return "testing-credentials"
    
    def connect(self, credentials):
        if credentials != self.VALID_CREDENTIALS:
            raise ChisubmitException("Invalid Testing Credentials")
    
    def disconnect(self, credentials):
        pass
    
    def init_course(self, course, fail_if_exists=True):
        pass            
    
    def deinit_course(self, course):
        pass
        
    def update_instructors(self, course):
        pass

    def update_graders(self, course):
        pass

    def create_team_repository(self, course, team, fail_if_exists=True, private=True):
        repo_path = self.__get_team_path(course, team)
        
        if os.path.exists(repo_path) and fail_if_exists:
            raise ChisubmitException("Repository %s already exists" % repo_path)
        
        repo = LocalGitRepo.create_repo(repo_path, bare=True)
        
    def update_team_repository(self, course, team):
        pass
    
    def exists_team_repository(self, course, team):
        repo_path = self.__get_team_path(course, team)
        
        return os.path.exists(repo_path)
    
    def get_repository_git_url(self, course, team):
        repo_path = self.__get_team_path(course, team)
        return repo_path
            
    def get_repository_http_url(self, course, team):
        return None
    
    def get_commit(self, course, team, commit_sha):
        repo_path = self.__get_team_path(course, team)
        repo = LocalGitRepo(repo_path)
        
        return repo.get_commit(commit_sha)        
        
    
    def create_submission_tag(self, course, team, tag_name, tag_message, commit_sha):
        repo_path = self.__get_team_path(course, team)
        repo = LocalGitRepo(repo_path)
        
        tag = repo.create_tag(tag_name, commit_sha, tag_message)
    
    def update_submission_tag(self, course, team, tag_name, tag_message, commit_sha):
        repo_path = self.__get_team_path(course, team)
        repo = LocalGitRepo(repo_path)
                
        tag = repo.create_tag(tag_name, commit_sha, tag_message, force=True)
            
    def get_submission_tag(self, course, team, tag_name):
        repo_path = self.__get_team_path(course, team)
        repo = LocalGitRepo(repo_path)
        
        tag = repo.get_tag(tag_name)
        
        if tag is None:
            return None
        else:
            return tag
    
    def delete_team_repository(self, course, team, fail_if_not_exists=True):
        pass
    
    def __get_team_path(self, course, team):
        return "%s/%s/%s.git" % (os.path.abspath(self.local_path), course.id, team.id)   
        




#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from chisubmit.client import CourseQualifiedApiObject, JSONObject
from chisubmit.common.utils import convert_datetime_to_utc
from chisubmit.client import session
from dateutil import parser
import json

class GradeComponent(JSONObject):
    
    _api_attrs = ('id', 'description', 'points', 'assignment_id')


class Assignment(CourseQualifiedApiObject):

    _api_attrs = ('name', 'deadline', 'id', 'course_id')
    _primary_key = 'id'    
    _has_many = {'grade_components': 'grade_components'}

    def __init__(self, *args, **kwargs):
        if kwargs.has_key("deadline"):
            if not hasattr(kwargs["deadline"], 'isoformat'):
                kwargs["deadline"] = convert_datetime_to_utc(parser.parse(kwargs['deadline']))
                
        super(Assignment, self).__init__(*args, **kwargs)
                
    def register(self, team_name=None, partners = []):
        url = self.url() + "/register"
        data = {}
        if team_name is not None:
            data["team_name"] = team_name
        data["partners"] = partners
        data = json.dumps(data)
        response = session.post(url, data=data)
        return response
    
    def submit(self, team_id, commit_sha, extensions, dry_run):
        url = self.url() + "/submit"
        data = {}
        data["team_id"] = team_id
        data["commit_sha"] = commit_sha
        data["extensions"] = extensions
        data["dry_run"] = dry_run
        data = json.dumps(data)
        response = session.post(url, data=data)
        return response    

    def cancel(self, team_id):
        url = self.url() + "/cancel"
        data = {}
        data["team_id"] = team_id
        data = json.dumps(data)
        response = session.post(url, data=data)
        return response    

    def get_grade_component(self, grade_component_id):
        gcs = [gc for gc in self.grade_components if gc.id == grade_component_id]
        
        if len(gcs) == 0:
            return None
        else:
            return gcs[0]

    def add_grade_component(self, gc):
        attrs = {}
        for attr in gc._api_attrs:
            attrs[attr] = getattr(gc, attr, None)
        data = json.dumps({'grade_components': {'add': [attrs]}})
        session.put(self.url(), data=data)

    def get_deadline(self):
        return self.deadline

    def get_grading_branch_name(self):
        return self.id + "-grading"



from unittest.suite import TestSuite

from . import test_courses
from . import test_assignments

test_cases = [test_courses, test_assignments]

def load_tests(loader, tests, pattern):
    suite = TestSuite()
    for test_module in test_cases:
        tests = loader.loadTestsFromModule(test_module)
        suite.addTests(tests)
    return suite



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import json
from chisubmit.client.assignment import Assignment
from chisubmit.client.team import Team
from chisubmit.client import session, ApiObject, JSONObject
from chisubmit.client.user import User

class CourseStudent(JSONObject):
    
    _api_attrs = ('dropped', 'repo_info')
    _has_one = {'user': ('student', User)}

class CourseInstructor(JSONObject):
    
    _api_attrs = ('repo_info',)
    _has_one = {'user': ('instructor', User)}

class CourseGrader(JSONObject):
    
    _api_attrs = ('repo_info','conflicts')
    _has_one = {'user': ('grader', User)}

    def get_conflicts(self):
        if self.conflicts is None or len(self.conflicts) == 0:
            return []
        else:
            return self.conflicts.split(",")
        
    def add_conflict(self, student_id):
        conflicts = self.get_conflicts()
        conflicts.append(student_id)
        conflicts_str = ",".join(conflicts)
        self.conflicts = conflicts_str

class Course(ApiObject):

    _api_attrs = ('id', 'name', 'options')
    _primary_key = 'id'    
    _updatable_attributes = ['name']
    
    _has_many = {'instructors': 'courses_instructors',
                 'graders': 'courses_graders',
                 'students': 'courses_students',
                 'assignments': 'assignments',
                 'teams': 'teams'}

    def add_student(self, student):
        attrs = {'course_id': self.id, 'student_id': student.id}
        data = json.dumps({'students': {'add': [attrs]}})
        session.put('courses/%s' % (self.id), data=data)

    def add_team(self, team):
        attrs = {'course_id': self.id, 'team_id': team.id}
        data = json.dumps({'teams': {'add': [attrs]}})
        session.put('courses/%s' % (self.id), data=data)

    def add_grader(self, grader):
        attrs = {'course_id': self.id, 'grader_id': grader.id}
        data = json.dumps({'graders': {'add': [attrs]}})
        session.put('courses/%s' % (self.id), data=data)
            
    def add_instructor(self, instructor):
        attrs = {'course_id': self.id, 'instructor_id': instructor.id}
        data = json.dumps({'instructors': {'add': [attrs]}})
        session.put('courses/%s' % (self.id), data=data)        
        
    def set_option(self, name, value):
        attrs = {'name': name, 'value': value}
        data = json.dumps({'options': {'update': [attrs]}})
        session.put('courses/%s' % (self.id), data=data)        

    def __set_repo_option(self, user_type, user_id, name, value):
        attrs = {'name': name, 'value': value}

        data = {"%ss" % user_type: 
                {
                 "update": [
                            {"%s_id" % user_type: user_id,
                             "repo_info": [attrs]}
                           ]
                }
               }        
        data = json.dumps(data)
        session.put('courses/%s' % (self.id), data=data) 
 
    def set_instructor_repo_option(self, instructor_id, name, value):
        self.__set_repo_option("instructor", instructor_id, name, value)

    def set_grader_repo_option(self, grader_id, name, value):
        self.__set_repo_option("grader", grader_id, name, value)

    def set_student_repo_option(self, student_id, name, value):
        self.__set_repo_option("student", student_id, name, value)
 
    def get_assignment(self, assignment_id):
        return Assignment.from_id(self.id, assignment_id)

    def get_student(self, student_id):
        ss = [s for s in self.students if s.user.id == student_id]
        
        if len(ss) == 1:
            return ss[0]
        else:
            return None             
        
    def set_student_dropped(self, student_id, dropped = True):
        data = {"students": 
                {
                 "update": [
                            {"student_id": student_id,
                             "dropped": 1}
                           ]
                }
               }
        data = json.dumps(data)
        session.put('courses/%s' % (self.id), data=data)         
        
    def add_grader_conflict(self, grader, student_id):
        grader.add_conflict(student_id)
        data = {"graders": 
                {
                 "update": [
                            {"grader_id": grader.user.id,
                             "conflicts": grader.conflicts}
                           ]
                }
               }
        from pprint import pprint
        pprint(data)
        data = json.dumps(data)
        session.put('courses/%s' % (self.id), data=data)           
        
    def get_team(self, team_id):
        return Team.from_id(self.id, team_id) 

    def get_teams(self):
        return Team.all(self.id)

    def search_team(self, search_term):
        teams = []
        for t in self.teams:
            if search_term in t._json:
                teams.append(t)

        return teams
    
    def has_grader(self, grader_id):        
        return self.get_grader(grader_id) is not None    
    
    def get_grader(self, grader_id):
        gs = [g for g in self.graders if g.user.id == grader_id]
        
        if len(gs) == 1:
            return gs[0]
        else:
            return None         



# Needed so we can import from the global "ldap" package
from __future__ import absolute_import

import ldap
from ldap.filter import filter_format
from chisubmit.backend.webapp.auth import Auth

global conn

class LDAPAuth(Auth):
    
    def __init__(self, server, ldap_server_uri, base_dn):
        super(LDAPAuth, self).__init__(server)

        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
        self.conn = ldap.initialize(ldap_server_uri)
        self.conn.protocol_version = ldap.VERSION3
        self.base_dn = base_dn
        self.server = server
        
    def check_auth(self, username, password):
        dn = self.__user_dn(username)
        try:
            self.conn.simple_bind_s(dn, password)
            return True
        except ldap.LDAPError:
            return False

    def __user_dn(self, uid):
        search_filter = filter_format("uid=%s", (uid,))
        return next(user[0]
                    for user in self.conn.search_s(self.base_dn,
                                              ldap.SCOPE_SUBTREE,
                                              search_filter, []))





import click

from chisubmit.repos.grading import GradingGitRepo
from chisubmit.common import CHISUBMIT_FAIL, CHISUBMIT_SUCCESS
from chisubmit.client.course import Course

from functools import update_wrapper

from dateutil.parser import parse
import operator


def pass_course(f):
    @click.pass_context
    def new_func(ctx, *args, **kwargs):
        course_id = ctx.obj["course_id"]
        course_specified = ctx.obj["course_specified"]

        if course_specified:
            course_obj = Course.from_id(course_id)
        else:
            if course_id is None:
                raise click.UsageError("No course specified with --course and there is no default course")
            else:
                course_obj = Course.from_id(course_id)

        if course_obj is None:
            raise click.BadParameter("Course '%s' does not exist" % course_id)

        ctx.obj["course_obj"] = course_obj

        return ctx.invoke(f, ctx.obj["course_obj"], *args, **kwargs)

    return update_wrapper(new_func, f)


class DateTimeParamType(click.ParamType):
    name = 'datetime'

    def convert(self, value, param, ctx):
        try:
            return parse(value)
        except ValueError:
            self.fail('"%s" is not a valid datetime string' % value, param, ctx)

DATETIME = DateTimeParamType()


def get_teams(course, assignment, only_ready_for_grading = False, grader = None, only = None):
    if only is not None:
        team = course.get_team(only)
        if team is None:
            print "Team %s does not exist"
            return None
        if not team.has_assignment(assignment.id):
            print "Team %s has not been assigned assignment %s" % (team.id, assignment.id)
            return None

        teams = [team]
    else:
        if only_ready_for_grading:
            teams = [t for t in course.teams if t.has_assignment_ready_for_grading(assignment)]
        else:
            teams = [t for t in course.teams if t.has_assignment(assignment.id)]

        if grader is not None:
            teams = [t for t in teams if getattr(t.get_assignment(assignment.id).grader, "id", None) == grader.user.id]

    teams.sort(key=operator.attrgetter("id"))

    return teams


def create_grading_repos(config, course, assignment, teams):
    repos = []

    for team in teams:
        repo = GradingGitRepo.get_grading_repo(config, course, team, assignment)

        if repo is None:
            print ("Creating grading repo for %s... " % team.id),
            repo = GradingGitRepo.create_grading_repo(config, course, team, assignment)
            repo.sync()

            repos.append(repo)

            print "done"
        else:
            print "Grading repo for %s already exists" % team.id

    return repos


def gradingrepo_push_grading_branch(config, course, team, assignment, to_students=False, to_staging=False):
    repo = GradingGitRepo.get_grading_repo(config, course, team, assignment)

    if repo is None:
        print "%s does not have a grading repository" % team.id
        return CHISUBMIT_FAIL
    
    if not repo.has_grading_branch():
        print "%s does not have a grading branch" % team.id
        return CHISUBMIT_FAIL

    if repo.is_dirty():
        print "Warning: %s grading repo has uncommitted changes." % team.id

    if to_students:
        repo.push_grading_branch_to_students()

    if to_staging:
        repo.push_grading_branch_to_staging()

    return CHISUBMIT_SUCCESS

def gradingrepo_pull_grading_branch(config, course, team, assignment, from_students=False, from_staging=False):
    assert(not (from_students and from_staging))
    repo = GradingGitRepo.get_grading_repo(config, course, team, assignment)

    if repo is None:
        print "%s does not have a grading repository" % team.id
        return CHISUBMIT_FAIL

    if repo.is_dirty():
        print "%s grading repo has uncommited changes. Cannot pull." % team.id
        return CHISUBMIT_FAIL

    if from_students:
        if not repo.has_grading_branch_staging():
            print "%s does not have a grading branch on students' repository" % team.id
        else:
            repo.pull_grading_branch_from_students()

    if from_staging:
        if not repo.has_grading_branch_staging():
            print "%s does not have a grading branch in staging" % team.id
        else:
            repo.pull_grading_branch_from_staging()

    return CHISUBMIT_SUCCESS





# Needed so we can import from the global "github" package
from __future__ import absolute_import

from github import Github, InputGitAuthor, NamedUser
from github.GithubException import GithubException

from chisubmit.repos import RemoteRepositoryConnectionBase, GitCommit, GitTag
from chisubmit.common import ChisubmitException
import pytz
from datetime import datetime
import sys

class GitHubConnection(RemoteRepositoryConnectionBase):

    def __init__(self, connection_string, staging):
        RemoteRepositoryConnectionBase.__init__(self, connection_string, staging)

        self.organization = None
        self.gh = None

    @staticmethod
    def get_server_type_name():
        return "GitHub"

    @staticmethod
    def get_connstr_mandatory_params():
        return ["github_organization"]

    @staticmethod
    def get_connstr_optional_params():
        return []

    def get_credentials(self, username, password, delete_repo = False):
        gh = Github(username, password)
        token = None

        try:
            u = gh.get_user()

            scopes = ['user', 'public_repo', 'repo', 'gist']
            note = "Created by chisubmit."

            if delete_repo:
                scopes.append("delete_repo")
                note += " Has delete permissions."
            
            auth = None
            for a in u.get_authorizations():
                if a.note == note:
                    auth = a
                    break
                
            if auth is None:
                existing = False
                auth = u.create_authorization(scopes = scopes, note = note)
            else:
                existing = True
            token = auth.token
        except GithubException as ge:
            if ge.status == 401:
                return None, False
            else:
                raise ChisubmitException("Unexpected error creating authorization token (%i: %s)" % (ge.status, ge.data["message"]), ge)

        return token, existing


    def connect(self, credentials):
        # Credentials are a GitHub access token

        self.gh = Github(credentials)

        try:
            self.organization = self.gh.get_organization(self.github_organization)
        except GithubException as ge:
            if ge.status == 401:
                raise ChisubmitException("Invalid Github Credentials", ge)
            elif ge.status == 404:
                raise ChisubmitException("Organization %s does not exist" % self.github_organization, ge)
            else:
                raise ChisubmitException("Unexpected error accessing organization %s (%i: %s)" % (self.github_organization, ge.status, ge.data["message"]), ge)


    def init_course(self, course, fail_if_exists=True):
        instructors_ghteam = self.__create_ghteam(self.__get_instructors_ghteam_name(course), [], "admin", fail_if_exists = fail_if_exists)
        graders_ghteam = self.__create_ghteam(self.__get_graders_ghteam_name(course), [], "push", fail_if_exists = fail_if_exists)

    def deinit_course(self, course):
        instructors_ghteam = self.__get_ghteam_by_name(self.__get_instructors_ghteam_name(course))
        graders_ghteam = self.__get_ghteam_by_name(self.__get_graders_ghteam_name(course))
        
        if instructors_ghteam is not None:
            instructors_ghteam.delete()
            
        if graders_ghteam is not None:
            graders_ghteam.delete()

    def update_instructors(self, course):
        instructors_ghteam = self.__get_ghteam_by_name(self.__get_instructors_ghteam_name(course))

        for instructor in course.instructors:
            self.__add_user_to_ghteam(self._get_user_git_username(instructor), instructors_ghteam)

        # TODO: Remove instructors that may have been removed


    def update_graders(self, course):
        graders_ghteam = self.__get_ghteam_by_name(self.__get_graders_ghteam_name(course))

        for grader in course.graders:
            self.__add_user_to_ghteam(self._get_user_git_username(grader), graders_ghteam)

        # TODO: Remove graders that may have been removed


    def create_team_repository(self, course, team, fail_if_exists=True, private=True):
        repo_name = self.__get_team_ghrepo_name(course, team)
        student_names = ", ".join(["%s %s" % (s.user.first_name, s.user.last_name) for s in team.students])
        repo_description = "%s: Team %s (%s)" % (course.name, team.id, student_names)
        github_instructors = self.__get_ghteam_by_name(self.__get_instructors_ghteam_name(course))
        github_graders = self.__get_ghteam_by_name(self.__get_graders_ghteam_name(course))

        students = [s for s in course.students if s.user.id in [ts.user.id for ts in team.students]]

        if not self.staging:
            github_students = []

            # Make sure users exist
            for s in students:
                username = self._get_user_git_username(s)
                github_student = self.__get_user(username)

                if github_student is None:
                    raise ChisubmitException("GitHub user '%s' does not exist " % (username))

                github_students.append(github_student)

        github_repo = self.__get_repository(repo_name)
        if github_repo is None:
            try:
                github_repo = self.organization.create_repo(repo_name, description=repo_description, private=private)
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception creating repository %s (%i: %s)" % (repo_name, ge.status, ge.data["message"]), ge)
        else:
            if fail_if_exists:
                raise ChisubmitException("Repository %s already exists" % repo_name)

        try:
            github_instructors.add_to_repos(github_repo)
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception adding repository to Instructors team (%i: %s)" % (ge.status, ge.data["message"]), ge)

        try:
            github_graders.add_to_repos(github_repo)
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception adding repository to Graders team (%i: %s)" % (ge.status, ge.data["message"]), ge)

        if not self.staging:
            team_name = self.__get_team_ghteam_name(course, team)

            github_team = self.__create_ghteam(team_name, [github_repo], "push", fail_if_exists)

            for github_student in github_students:
                try:
                    self.__pygithub_add_membership(github_team, github_student)
                except GithubException as ge:
                    raise ChisubmitException("Unexpected exception adding user %s to team (%i: %s)" % (username, ge.status, ge.data["message"]), ge)


    def update_team_repository(self, course, team):
        github_team = self.__get_ghteam_by_name(self.__get_team_ghteam_name(course, team))

        students = [s for s in course.students if s.user.id in [ts.user.id for ts in team.students]]

        for s in students:
            username = self._get_user_git_username(s)
            github_student = self.__get_user(username)
            try:
                self.__pygithub_add_membership(github_team, github_student)
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception adding user %s to team (%i: %s)" % (username, ge.status, ge.data["message"]))

        #TODO: Remove students that may have been removed from team

    def exists_team_repository(self, course, team):
        r = self.__get_repository(self.__get_team_ghrepo_name(course, team))

        return (r is not None)


    def get_repository_git_url(self, course, team):
        orgname = self.github_organization
        reponame = self.__get_team_ghrepo_name(course, team)
        return "git@github.com:%s/%s.git" % (orgname, reponame)

    def get_repository_http_url(self, course, team):
        orgname = self.github_organization
        reponame = self.__get_team_ghrepo_name(course, team)
        return "https://github.com/%s/%s" % (orgname, reponame)


    # Return new commit object
    def get_commit(self, course, team, commit_sha):
        try:
            github_repo = self.organization.get_repo(self.__get_team_ghrepo_name(course, team))
            gh_commit = github_repo.get_commit(commit_sha)
            
            commit = GitCommit(gh_commit.commit.sha, gh_commit.commit.message, 
                 gh_commit.commit.author.name, gh_commit.commit.author.email, gh_commit.commit.author.date,
                 gh_commit.commit.committer.name, gh_commit.commit.committer.email, gh_commit.commit.committer.date)

            return commit
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error when fetching commit %s (%i: %s)" % (commit_sha, ge.status, ge.data["message"]), ge)


    def create_submission_tag(self, course, team, tag_name, tag_message, commit_sha):
        github_repo = self.organization.get_repo(self.__get_team_ghrepo_name(course, team))

        commit = self.get_commit(course, team, commit_sha)

        this_user = self.gh.get_user()

        if this_user.name is None:
            user_name = "Team %s" % team.id
        else:
            user_name = this_user.name

        if this_user.email is None:
            user_email = "unspecified@example.org"
        else:
            user_email = this_user.email

        tz = pytz.timezone("America/Chicago")
        dt = tz.localize(datetime.now().replace(microsecond=0))
        iu = InputGitAuthor(user_name, user_email, dt.isoformat())

        tag = github_repo.create_git_tag(tag_name, tag_message, commit.sha, "commit", iu)
        github_repo.create_git_ref("refs/tags/" + tag.tag, tag.sha)


    def update_submission_tag(self, course, team, tag_name, tag_message, commit_sha):
        submission_tag_ref = self.__get_submission_tag_ref(course, team, tag_name)
        submission_tag_ref.delete()

        self.create_submission_tag(course, team, tag_name, tag_message, commit_sha)


    # Return a new "tag" object
    def get_submission_tag(self, course, team, tag_name):
        github_repo = self.organization.get_repo(self.__get_team_ghrepo_name(course, team))

        submission_tag_ref = self.__get_submission_tag_ref(course, team, tag_name)

        if submission_tag_ref is None:
            return None

        gh_tag = github_repo.get_git_tag(submission_tag_ref.object.sha)
        
        tag = GitTag(name = gh_tag.tag,
                     commit = self.get_commit(course, team, gh_tag.object.sha))

        return tag


    def delete_team_repository(self, course, team, fail_if_not_exists=True):
        ghrepo_name = self.__get_team_ghrepo_name(course, team)
        github_repo = self.__get_repository(ghrepo_name)
        
        if github_repo is None:
            if fail_if_not_exists:
                raise ChisubmitException("Trying to delete a repository that doesn't exist (%s)" % (ghrepo_name))
            else:
                return 

        try:
            github_repo.delete()
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception deleting repository %s (%i: %s)" % (ghrepo_name, ge.status, ge.data["message"]), ge)

        github_team_name = self.__get_team_ghteam_name(course, team)
        github_team = self.__get_ghteam_by_name(github_team_name)

        try:
            github_team.delete()
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception deleting team %s (%i: %s)" % (github_team_name, ge.status, ge.data["message"]), ge)

    def __get_instructors_ghteam_name(self, course):
        return "%s-instructors" % course.id

    def __get_graders_ghteam_name(self, course):
        return "%s-graders" % course.id

    def __get_team_ghteam_name(self, course, team):
        return "%s-%s" % (course.id, team.id)

    def __get_team_ghrepo_name(self, course, team):
        return "%s-%s" % (course.id, team.id)

    def __create_ghteam(self, team_name, repos, permissions, fail_if_exists = True):
        github_team = self.__get_ghteam_by_name(team_name)

        if github_team is not None:
            if fail_if_exists:
                raise ChisubmitException("Team %s already exists." % team_name)
            else:
                return github_team
        else:
            try:
                github_team = self.organization.create_team(team_name, repos, permissions)
                return github_team
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception creating team %s (%i: %s)" % (team_name, ge.status, ge.data["message"]), ge)

    def __add_user_to_ghteam(self, github_id, ghteam):
        github_user = self.__get_user(github_id)

        if github_user is None:
            raise ChisubmitException("GitHub user '%s' does not exist " % github_id)

        try:
            self.__pygithub_add_membership(ghteam, github_user)
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception adding user %s to team (%i: %s)" % (github_id, ge.status, ge.data["message"]), ge)

    # This is a workaround until PyGithub fixes this:
    # https://github.com/jacquev6/PyGithub/issues/280
    def __pygithub_add_membership(self, team, member):
        """
        :calls: `PUT /teams/:id/memberships/:user <http://developer.github.com/v3/orgs/teams>`_
        :param member: :class:`github.Nameduser.NamedUser`
        :rtype: None
        """
        assert isinstance(member, NamedUser.NamedUser), member
        headers, data = team._requester.requestJsonAndCheck(
            "PUT",
            team.url + "/memberships/" + member._identity
        )

    def __add_user_to_ghteam_by_name(self, github_id, ghteam_name):
        ghteam = self.__get_ghteam_by_name(ghteam_name)

        if ghteam is None:
            raise ChisubmitException("GitHub team '%s' does not exist " % ghteam_name)

        self.__add_user_to_ghteam(github_id, ghteam)


    def __get_user(self, username):
        try:
            user = self.gh.get_user(username)
            return user
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error with user %s (%i: %s)" % (username, ge.status, ge.data["message"]), ge)


    def __get_repository(self, repository_name):
        try:
            repository = self.organization.get_repo(repository_name)
            return repository
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error with repository %s (%i: %s)" % (repository_name, ge.status, ge.data["message"]), ge)


    def __get_ghteam_by_name(self, team_name):
        try:
            teams = self.organization.get_teams()
            for t in teams:
                if t.name == team_name:
                    return t
            return None
        except GithubException as ge:
            raise ChisubmitException("Unexpected error with team %s (%i: %s)" % (team_name, ge.status, ge.data["message"]), ge)


    def __get_submission_tag_ref(self, course, team, tag_name):
        github_repo = self.organization.get_repo(self.__get_team_ghrepo_name(course, team))

        try:
            submission_tag_ref = github_repo.get_git_ref("tags/" + tag_name)
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error when fetching tag %s (%i: %s)" % (tag_name, ge.status, ge.data["message"]), ge)

        return submission_tag_ref




from chisubmit.tests.fixtures import users_and_courses, complete_course
import unittest

#class UsersAndCoursesFixtureCheck(ChisubmitFixtureTestCase, unittest.TestCase):
#        
#    FIXTURE = users_and_courses 
#    
#    
#class CompleteCourseFixtureCheck(ChisubmitFixtureTestCase, unittest.TestCase):
#        
#    FIXTURE = complete_course     


from chisubmit.backend.webapp.api import db
from chisubmit.backend.webapp.api.models.json import Serializable
from chisubmit.backend.webapp.api.types import JSONEncodedDict, UTCDateTime
from chisubmit.backend.webapp.api.assignments.models import GradeComponent
from sqlalchemy.ext.associationproxy import association_proxy
from datetime import timedelta
from chisubmit.common.utils import get_datetime_now_utc

class Team(Serializable, db.Model):
    __tablename__ = 'teams'
    id = db.Column(db.Unicode, primary_key=True)
    extensions = db.Column(db.Integer, default=0)    
    repo_info = db.Column(JSONEncodedDict, default={})
    extras = db.Column(JSONEncodedDict, default={})
    active = db.Column('active', db.Boolean,
                       server_default='1', nullable=False)
    course_id = db.Column('course_id',
                          db.Integer, db.ForeignKey('courses.id'), primary_key=True)
    students = association_proxy('students_teams', 'student')
    assignments = association_proxy('assignments_teams', 'assignment')

    default_fields = ['extensions', 'repo_info', 'active', 'course_id', 'extras',
                      'students_teams', 'assignments_teams', 'grades']
    readonly_fields = ['students', 'assignments', 'grades', 'extras']
    
    @staticmethod
    def from_id(course_id, team_id):
        return Team.query.filter_by(course_id=course_id,id=team_id).first()        
    
    @staticmethod
    def find_teams_with_students(course_id, students):
        q = Team.query.filter(Team.course_id == course_id,
                Team.students.any(
                    StudentsTeams.student_id.in_(
                        [s.id for s in students])))
        
        return q.all()       
    
    def get_extensions_used(self):
        extensions = 0
        for at in self.assignments_teams:
            extensions += at.extensions_used
        return extensions 
        
    def get_extensions_available(self, extension_policy):
        from chisubmit.backend.webapp.api.courses.models import CoursesStudents

        if extension_policy == "per_team":
            return self.extensions - self.get_extensions_used()    
        elif extension_policy == "per_student":
            student_extensions_available = []
            for student in self.students:
                cs = CoursesStudents.from_id(self.course_id, student.id)
                a = cs.get_extensions_available()
                student_extensions_available.append(a)
            return min(student_extensions_available)
        else:
            return 0        

class StudentsTeams(Serializable, db.Model):
    STATUS_UNCONFIRMED = 0
    STATUS_CONFIRMED = 1
    
    __tablename__ = 'students_teams'
    status = db.Column(db.Integer, nullable=False, server_default='0')
    student_id = db.Column('student_id',
                           db.Unicode,
                           db.ForeignKey('users.id'),
                           primary_key=True)
    team_id = db.Column('team_id',
                        db.Integer,
                        primary_key=True)
    course_id = db.Column('course_id', 
                          db.Integer, 
                          primary_key=True)
    student = db.relationship("User")
    default_fields = ['status', 'student']
    readonly_fields = ['student', 'team']
    team = db.relationship("Team",
                           backref=db.backref("students_teams",
                                              cascade="all, delete-orphan"))
    __table_args__ = (db.ForeignKeyConstraint([team_id, course_id],
                                              [Team.id, Team.course_id]),
                      {})

class AssignmentsTeams(Serializable, db.Model):
    __tablename__ = 'assignments_teams'

    assignment_id = db.Column('assignment_id',
                           db.Integer, primary_key=True)
    team_id = db.Column('team_id',
                        db.Unicode, primary_key=True)
    course_id = db.Column('course_id', 
                          db.Integer, primary_key=True)

    extensions_used = db.Column(db.Integer, default = 0)
    commit_sha = db.Column(db.Unicode)
    submitted_at = db.Column(UTCDateTime)
    grader_id = db.Column('grader_id',
                          db.Integer,
                          db.ForeignKey('users.id'))
    penalties = db.Column(JSONEncodedDict, default={})


    team = db.relationship("Team",
                           backref=db.backref("assignments_teams",
                                              cascade="all, delete-orphan"))
    assignment = db.relationship("Assignment")
    grader = db.relationship("User")
    
    default_fields = ['extensions_used', 'commit_sha', 'submitted_at', 'assignment_id', 
                      'grades', 'penalties', 'grader']
    readonly_fields = ['team', 'grader', 'grades']
    __table_args__ = (db.ForeignKeyConstraint([team_id, course_id],
                                              [Team.id, Team.course_id]),
                      db.ForeignKeyConstraint([assignment_id, course_id],
                                              ["assignments.id", "assignments.course_id"]),
                      {})
    

    @staticmethod
    def from_id(course_id, team_id, assignment_id):
        return AssignmentsTeams.query.filter_by(course_id=course_id,
                                                team_id=team_id,
                                                assignment_id=assignment_id).first()
                                                
    def get_grade(self, gc_id):
        grades = [g for g in self.grades if g.grade_component_id == gc_id]
        if len(grades) == 0:
            return None
        else:
            return grades[0]           
        
    def is_ready_for_grading(self):
        if self.submitted_at is None:
            return False
        else:
            now = get_datetime_now_utc()
            deadline = self.assignment.deadline + timedelta(days=self.extensions_used)
            
            if now > deadline:
                return True
            else:
                return False
                                   
                                    
class Grade(Serializable, db.Model):
    __tablename__ = 'grades'

    course_id = db.Column('course_id', db.Integer, primary_key=True)
    assignment_id = db.Column('assignment_id', db.Integer, primary_key=True)
    grade_component_id = db.Column('grade_component_id',
                                     db.Unicode, primary_key=True)
    team_id = db.Column('team_id', db.Integer, primary_key=True)

    points = db.Column(db.Float, nullable=False)
    
    grade_component = db.relationship('GradeComponent')
    team = db.relationship('AssignmentsTeams', backref="grades")
    
    default_fields = ['points', 'grade_component_id']
    readonly_fields = ['course_id', 'team_id', 'grade_component']
 
    __table_args__ = (db.ForeignKeyConstraint([team_id, assignment_id, course_id],
                                              [AssignmentsTeams.team_id, AssignmentsTeams.assignment_id, AssignmentsTeams.course_id]),
                      db.ForeignKeyConstraint([grade_component_id, assignment_id, course_id],
                                              [GradeComponent.id, GradeComponent.assignment_id, GradeComponent.course_id]),
                      {})
        


from wtforms import Form, StringField, IntegerField
from wtforms.fields import FormField, FieldList
from wtforms.validators import Length, InputRequired, NumberRange, Optional
from chisubmit.backend.webapp.api.assignments.forms import CreateAssignmentInput
from chisubmit.backend.webapp.api.teams.forms import CreateTeamInput
from wtforms.fields.core import BooleanField


class SetOptionInput(Form):
    name = StringField(validators=[InputRequired()])
    value = StringField(validators=[InputRequired()])


class AddTeamsInput(Form):
    add = FieldList(FormField(CreateTeamInput))


class AddAssignmentsInput(Form):
    add = FieldList(FormField(CreateAssignmentInput))


class LinkGraderInput(Form):
    course_id = StringField(validators=[InputRequired()])
    grader_id = StringField(validators=[InputRequired()])


class LinkStudentInput(Form):
    course_id = StringField(validators=[InputRequired()])
    student_id = StringField(validators=[InputRequired()])

class UpdateStudentInput(Form):
    student_id = StringField(default = None)
    dropped = BooleanField(validators=[Optional()])
    repo_info = FieldList(FormField(SetOptionInput))

class UpdateInstructorInput(Form):
    instructor_id = StringField(default = None)
    repo_info = FieldList(FormField(SetOptionInput))

class UpdateGraderInput(Form):
    grader_id = StringField(default = None)
    repo_info = FieldList(FormField(SetOptionInput))
    conflicts = StringField(validators=[Optional()])


class LinkInstructorInput(Form):
    course_id = StringField(validators=[InputRequired()])
    instructor_id = StringField(validators=[InputRequired()])


class AddGradersInput(Form):
    add = FieldList(FormField(LinkGraderInput))
    update = FieldList(FormField(UpdateGraderInput))


class AddStudentsInput(Form):
    add = FieldList(FormField(LinkStudentInput))
    update = FieldList(FormField(UpdateStudentInput))


class AddInstructorsInput(Form):
    add = FieldList(FormField(LinkInstructorInput))
    update = FieldList(FormField(UpdateInstructorInput))


class UpdateOptionInput(Form):
    update = FieldList(FormField(SetOptionInput))

class CreateCourseInput(Form):
    # FIXME 10DEC14: take the primary key out of the user's hands
    id = StringField(validators=[Length(max=36, min=2), InputRequired()])
    name = StringField(validators=[Length(max=36, min=2), InputRequired()])


class UpdateCourseInput(Form):
    # FIXME 10DEC14: take the primary key out of the user's hands
    id = StringField(validators=[Length(max=36, min=2), Optional()])
    name = StringField(validators=[Length(max=36, min=2), Optional()])
    assignments = FormField(AddAssignmentsInput)
    instructors = FormField(AddInstructorsInput)
    students = FormField(AddStudentsInput)
    teams = FormField(AddTeamsInput)
    graders = FormField(AddGradersInput)
    options = FormField(UpdateOptionInput)


class SearchCourseStudentInput(Form):
    course_id = StringField(InputRequired())
    student_id = StringField(InputRequired())



import click

import os.path
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.cli.common import create_grading_repos,\
    gradingrepo_push_grading_branch, gradingrepo_pull_grading_branch, get_teams
from chisubmit.repos.grading import GradingGitRepo
from chisubmit.rubric import RubricFile, ChisubmitRubricException
from chisubmit.cli.common import pass_course
from chisubmit.cli.shared.course import shared_course_get_git_credentials


@click.group()
@click.pass_context
def grader(ctx):
    pass

@click.command(name="create-local-grading-repos")
@click.argument('grader_id', type=str)
@click.argument('assignment_id', type=str)
@pass_course
@click.pass_context
def grader_create_local_grading_repos(ctx, course, grader_id, assignment_id):
    grader = course.get_grader(grader_id)
    if not grader:
        print "Grader %s does not exist" % grader_id
        ctx.exit(CHISUBMIT_FAIL)

    assignment = course.get_assignment(assignment_id)
    if not assignment:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, grader = grader)

    if not teams:
        print "No teams found"
        ctx.exit(CHISUBMIT_FAIL)

    repos = create_grading_repos(ctx.obj['config'], course, assignment, teams)

    if not repos:
        print "There was some kind of problem creating the grading repos."
        ctx.exit(CHISUBMIT_FAIL)

    for repo in repos:
        repo.set_grader_author()

    for team in teams:
        print ("Pulling grading branch for team %s... " % team.id),
        gradingrepo_pull_grading_branch(ctx.obj['config'], course, team, assignment, from_staging = True)
        print "done"
        
    return CHISUBMIT_SUCCESS


@click.command(name="validate-rubrics")
@click.argument('grader_id', type=str)
@click.argument('assignment_id', type=str)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def grader_validate_rubrics(ctx, course, grader_id, assignment_id, only):
    grader = course.get_grader(grader_id)
    if not grader:
        print "Grader %s does not exist" % grader_id
        ctx.exit(CHISUBMIT_FAIL)

    assignment = course.get_assignment(assignment_id)
    if not assignment:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, grader = grader, only = only)
    
    for team in teams:

        repo = GradingGitRepo.get_grading_repo(ctx.obj['config'], course, team, assignment)
        if not repo:
            print "Repository for %s does not exist" % (team.id)
            ctx.exit(CHISUBMIT_FAIL)
    
        rubricfile = repo.repo_path + "/%s.rubric.txt" % assignment.id
    
        if not os.path.exists(rubricfile):
            print "Repository for %s does not exist have a rubric for assignment %s" % (team.id, assignment.id)
            ctx.exit(CHISUBMIT_FAIL)
    
        try:
            RubricFile.from_file(open(rubricfile), assignment)
            print "%s: Rubric OK." % team.id
        except ChisubmitRubricException, cre:
            print "%s: Rubric ERROR: %s" % (team.id, cre.message)

    return CHISUBMIT_SUCCESS


@click.command(name="push-grading-branches")
@click.argument('grader_id', type=str)
@click.argument('assignment_id', type=str)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def grader_push_grading_branches(ctx, course, grader_id, assignment_id, only):
    grader = course.get_grader(grader_id)
    if not grader:
        print "Grader %s does not exist" % grader_id
        ctx.exit(CHISUBMIT_FAIL)

    assignment = course.get_assignment(assignment_id)
    if not assignment:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, grader = grader, only = only, only_ready_for_grading=True)

    if not teams:
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        print "Pushing grading branch for team %s... " % team.id
        gradingrepo_push_grading_branch(ctx.obj['config'], course, team, assignment, to_staging = True)

    return CHISUBMIT_SUCCESS

@click.command(name="pull-grading-branches")
@click.argument('grader_id', type=str)
@click.argument('assignment_id', type=str)
@click.option('--only', type=str)
@pass_course
@click.pass_context
def grader_pull_grading_branches(ctx, course, grader_id, assignment_id, only):
    grader = course.get_grader(grader_id)
    if not grader:
        print "Grader %s does not exist" % grader_id
        ctx.exit(CHISUBMIT_FAIL)

    assignment = course.get_assignment(assignment_id)
    if not assignment:
        print "Assignment %s does not exist" % assignment_id
        ctx.exit(CHISUBMIT_FAIL)

    teams = get_teams(course, assignment, grader = grader, only = only, only_ready_for_grading=True)

    if not teams:
        ctx.exit(CHISUBMIT_FAIL)

    for team in teams:
        print "Pulling grading branch for team %s... " % team.id
        gradingrepo_pull_grading_branch(ctx.obj['config'], course, team, assignment, from_staging = True)

    return CHISUBMIT_SUCCESS

grader.add_command(grader_create_local_grading_repos)
grader.add_command(grader_validate_rubrics)
grader.add_command(grader_push_grading_branches)
grader.add_command(grader_pull_grading_branches)
grader.add_command(shared_course_get_git_credentials)




from chisubmit.tests.common import ChisubmitMultiTestCase, ChisubmitTestCase
from chisubmit.client.course import Course
from chisubmit.tests.fixtures import users_and_courses, complete_course
from chisubmit.client.assignment import Assignment

import unittest
from chisubmit.client.session import BadRequestError
        
        
class EmptyCourse(ChisubmitMultiTestCase):
    
    FIXTURE = users_and_courses
        
    def test_create_assignment(self):
        
        self.get_test_client({"id":"admin", "api_key":"admin"})
        
        assignment = Assignment(id = "pa1",
                                name = "Programming Assignment 1",
                                deadline = "2042-01-21T20:00",
                                course_id = "cmsc40100")        
        
        
class CompleteCourse(ChisubmitMultiTestCase):
    
    FIXTURE = complete_course
        
    def test_get_assignments(self):
        
        self.get_test_client({"id":"student1", "api_key":"student1"})
        
        assignments = Assignment.all("cmsc40100")
                
    def test_get_assignment(self):
        
        self.get_test_client({"id":"student1", "api_key":"student1"})
        
        assignment = Assignment.from_id("cmsc40100", "pa1")
        
        
class Registration(ChisubmitTestCase):
    
    FIXTURE = complete_course
        
    def test_single_registration1(self):
        
        self.get_test_client({"id":"student1", "api_key":"student1"})
        
        course = self.FIXTURE["courses"]["cmsc40100"]
        assignment = course["assignments"]["pa1"] 
        
        a = Assignment.from_id(course["id"], assignment["id"])
        self.assertEquals(a.name, assignment["name"])
                
        r = a.register(partners=["student2", "student3"]) 
        
    def test_single_registration2(self):
        
        self.get_test_client({"id":"student1", "api_key":"student1"})
        
        course = self.FIXTURE["courses"]["cmsc40100"]
        assignment = course["assignments"]["pa1"] 
        
        a = Assignment.from_id(course["id"], assignment["id"])
        self.assertEquals(a.name, assignment["name"])
                
        r = a.register(partners=["student2", "student3"])         
            
        self.get_test_client({"id":"student2", "api_key":"student2"})
        r = a.register(partners=["student1", "student3"])         

        self.get_test_client({"id":"student3", "api_key":"student3"})
        r = a.register(partners=["student1", "student2"])         

            
    def test_team_multiple_registration1(self):
        
        self.get_test_client({"id":"student1", "api_key":"student1"})
        
        course = self.FIXTURE["courses"]["cmsc40100"]
        assignment1 = course["assignments"]["pa1"] 
        assignment2 = course["assignments"]["pa2"] 
        
        a1 = Assignment.from_id(course["id"], assignment1["id"])
        self.assertEquals(a1.name, assignment1["name"])

        a2 = Assignment.from_id(course["id"], assignment2["id"])
        self.assertEquals(a2.name, assignment2["name"])
                
        r1 = a1.register(partners=["student2"]) 
        r2 = a2.register(partners=["student2"]) 
     

        
                              